from geopytool.ImportDependence import *
from geopytool.CustomClass import *

LocationOfMySelf=os.path.dirname(__file__)

print(LocationOfMySelf,'Import Denpendence')

fpath = LocationOfMySelf+('/wqy.ttf')

font = ft2font.FT2Font(fpath)
fprop = font_manager.FontProperties(fname=fpath)

ttfFontProp = ttfFontProperty(font)
fontprop = font_manager.FontProperties(family='sans-serif',
                            #name=ap.fontprop.name,
                            size=9,
                            fname=ttfFontProp.fname,
                            stretch=ttfFontProp.stretch,
                            style=ttfFontProp.style,
                            variant=ttfFontProp.variant,
                            weight=ttfFontProp.weight)

plt.rcParams['svg.fonttype'] = 'none'
plt.rcParams['pdf.fonttype'] = 'truetype'
plt.rcParams['axes.unicode_minus']=False

class QAPF(AppForm, Tool):

    infotext ='Q = quartz, A = alkali feldspar, P = plagioclase and F = feldspathoid.\nOnly for rocks in which the mafic mineral content, M, is greater than 90%.'

    exptext='''
            1a: quartzolite,
            1b: quartz-rich granitoid,		
            2: alkali feldspar granite,
            3a: (syeno granite),
            3b: (monzo granite),
            4: granodiorite,
            5: tonalite,		
            6*: quartz alkali feldspar syenite,
            7*: quartz syenite,
            8*: quartz monzonite,
            9*: quartz monzodiorite quartz monzogabbro,
            10*: quartz diorite quartz gabbro  quartz anorthosite,		
            6: alkali feldspar syenite,
            7: syenite,
            8: monzonite,
            9: monzodiorite monzogabbro,
            10: diorite gabbro anorthosite,		
            6\': foid-bearing alkali feldspar syenite,
            7\': foid-bearing syenite,
            8\': foid-bearing monzonite,
            9\': foid-bearing monzodiorite foid-bearing monzogabbro,
            10\': foid-bearing diorite foid-bearing gabbro foid-bearing anorthosite,		
            11: foid syenite,
            12: foid monzosyenite,
            13: foid monzodiorite foid monzogabbro,
            14: foid diorite foid gabbro,
            15: foidolite
            '''


    #infotext = infotext + exptext


    reference = 'Reference: Maitre, R. W. L., Streckeisen, A., Zanettin, B., Bas, M. J. L., Bonin, B., and Bateman, P., 2004, Igneous Rocks: A Classification and Glossary of Terms: Cambridge University Press, v. -1, no. 70, p. 93–120.'
    _df = pd.DataFrame()
    _changed = False

    xlabel = r''
    ylabel = r''

    Tags = []

    Label = [u'Q', u'A', u'P', u'F']
    LabelPosition = [(48, 50 * np.sqrt(3) + 1),
                     (-6, -1),
                     (104, -1),
                     (49, -50 * np.sqrt(3) - 4)]

    Labels = ['quartzolite',

              'quartz-rich\ngranitoid',

              'granite',

              'alkali\nfeldspar\ngranite',
              '(syeno\ngranite)',
              '(monzo\ngranite)',
              'granodiorite',
              'tonalite',

              'quartz\nalkali\nfeldspar\nsyenite',
              'quartz\nsyenite',
              'quartz\nmonzonite',
              'quartz\nmonzodiorite\nquartz\nmonzogabbro',
              'quartz\ndiorite\nquartz gabbro\n quartz\nanorthosite',

              'alkali\nfeldspar\nsyenite',
              'syenite',
              'monzonite',
              'monzodiorite\nmonzogabbro',
              'diorite\ngabbro\nanorthosite',

              'foid-bearing\nalkali\nfeldspar\nsyenite',
              'foid-bearing\nsyenite',
              'foid-bearing\nmonzonite',
              'foid-bearing\nmonzodiorite\nfoid-bearing\nmonzogabbro',
              'foid-bearing\ndiorite\nfoid-bearing gabbro\nfoid-bearing\nanorthosite',

              'foid\nsyenite',
              'foid\nmonzosyenite',
              'foid\nmonzodiorite\nfoid\nmonzogabbro',
              'foid\ndiorite\nfoid\ngabbro',
              'foidolite']

    Locations = [(5, 5, 95),

                 (10, 10, 80),

                 (35, 15, 50),

                 (45, 5, 50),
                 (45, 25, 30),
                 (35, 35, 30),
                 (25, 45, 30),
                 (5, 45, 50),

                 (85, 5, 10),
                 (75, 15, 10),
                 (45, 45, 10),
                 (15, 75, 10),
                 (5, 85, 10),

                 (93, 5, 1),
                 (83, 15, 1),
                 (53, 53, 1),
                 (15, 83, 1),
                 (5, 93, 1),

                 (95, 3, -8),
                 (75, 23, -8),
                 (49, 49, -8),
                 (23, 75, -8),
                 (3, 95, -8),

                 (63, 7, -30),
                 (50, 20, -30),
                 (20, 50, -30),
                 (7, 63, -30),
                 (10, 10, -80)]

    Offset = [(-30, 0),

              (-30, 0),

              (-20, 0),

              (-70, 30),
              (-50, 30),
              (-30, 0),
              (0, 0),
              (30, 20),

              (-70, 15),
              (-10, 0),
              (-40, 0),
              (-50, -5),
              (30, 15),

              (-80, 5),
              (0, 0),
              (-40, 0),
              (-50, -5),
              (60, 5),

              (-80, -15),
              (-40, 0),
              (-40, 0),
              (-20, -15),
              (50, -30),

              (-80, 0),
              (-40, 0),
              (-40, 0),
              (60, 0),
              (-30, 0)]

    LocationAreas0 = [[[50.0, 86.60254037844386], [45.0, 77.94228634059948], [55.0, 77.94228634059948]],
                         [[45.0, 77.94228634059948], [30.0, 51.96152422706631], [70.0, 51.96152422706631],
                          [55.0, 77.94228634059948]],
                         [[30.0, 51.96152422706631], [10.0, 17.32050807568877], [18.0, 17.32050807568877],
                          [34.0, 51.96152422706631]],
                         [[34.0, 51.96152422706631], [18.0, 17.32050807568877], [38.0, 17.32050807568877],
                          [44.0, 51.96152422706631]],
                         [[44.0, 51.96152422706631], [38.0, 17.32050807568877], [62.0, 17.32050807568877],
                          [56.0, 51.96152422706631]],
                         [[56.0, 51.96152422706631], [62.0, 17.32050807568877], [82.0, 17.32050807568877],
                          [66.0, 51.96152422706631]],
                         [[66.0, 51.96152422706631], [82.0, 17.32050807568877], [90.0, 17.32050807568877],
                          [70.0, 51.96152422706631]],
                         [[10.0, 17.32050807568877], [2.5, 4.330127018922193], [12.0, 4.330127018922193],
                          [18.0, 17.32050807568877]],
                         [[18.0, 17.32050807568877], [12.0, 4.330127018922193], [35.75, 4.330127018922193],
                          [38.0, 17.32050807568877]],
                         [[38.0, 17.32050807568877], [35.75, 4.330127018922193], [64.25, 4.330127018922193],
                          [62.0, 17.32050807568877]],
                         [[62.0, 17.32050807568877], [64.25, 4.330127018922193], [88.34951456310678, 4.204006814487566],
                          [82.0, 17.32050807568877]],
                         [[88.34951456310678, 4.204006814487566], [82.0, 17.32050807568877], [97.5, 4.330127018922193],
                          [90.0, 17.32050807568877]],
                         [[2.5, 4.330127018922193], [0.0, 0.0], [10.0, 0.0], [11.650485436893202, 4.204006814487566]],
                         [[11.650485436893202, 4.204006814487566], [10.0, 0.0], [35.0, 0.0],
                          [35.75, 4.330127018922193]],
                         [[35.75, 4.330127018922193], [35.0, 0.0], [65.0, 0.0], [64.25, 4.330127018922193]],
                         [[64.25, 4.330127018922193], [65.0, 0.0], [90.0, 0.0], [88.34951456310678, 4.204006814487566]],
                         [[88.34951456310678, 4.204006814487566], [90.0, 0.0], [100.0, 0.0], [97.5, 4.330127018922193]],
                         [[0.0, 0.0], [5.0, -8.660254037844386], [14.0, -8.660254037844386], [10.0, 0.0]],
                         [[10.0, 0.0], [14.0, -8.660254037844386], [36.42857142857143, -8.247860988423225],
                          [35.0, 0.0]],
                         [[35.0, 0.0], [36.42857142857143, -8.247860988423225], [63.57142857142858, -8.247860988423225],
                          [65.0, 0.0]],
                         [[65.0, 0.0], [63.57142857142858, -8.247860988423225], [86.0, -8.660254037844386],
                          [90.0, 0.0]],
                         [[90.0, 0.0], [86.0, -8.660254037844386], [95.0, -8.660254037844386], [100.0, 0.0]],
                         [[5.0, -8.660254037844386], [30.0, -51.96152422706631], [34.0, -51.96152422706631],
                          [14.0, -8.660254037844386]],
                         [[14.0, -8.660254037844386], [34.0, -51.96152422706631], [50.0, -51.96152422706631],
                          [50.0, -8.660254037844386]],
                         [[50.0, -8.660254037844386], [50.0, -51.96152422706631], [66.0, -51.96152422706631],
                          [86.0, -8.660254037844386]],
                         [[86.0, -8.660254037844386], [66.0, -51.96152422706631], [70.0, -51.96152422706631],
                          [95.0, -8.660254037844386]],
                         [[30.0, -51.96152422706631], [50.0, -86.60254037844386], [70.0, -51.96152422706631]]]

    ItemNames0 = [
        '1a: quartzolite',
        '1b: quartz-rich granitoid',
        '2: alkali feldspar granite',
        '3a: (syeno granite)',
        '3b: (monzo granite)',
        '4: granodiorite',
        '5: tonalite',
        '6*: quartz alkali feldspar syenite',
        '7*: quartz syenite',
        '8*: quartz monzonite',
        '9*: quartz monzodiorite quartz monzogabbro',
        '10*: quartz diorite quartz gabbro  quartz anorthosite',
        '6: alkali feldspar syenite',
        '7: syenite',
        '8: monzonite',
        '9: monzodiorite monzogabbro',
        '10: diorite gabbro anorthosite',
        '6\': foid-bearing alkali feldspar syenite',
        '7\': foid-bearing syenite',
        '8\': foid-bearing monzonite',
        '9\': foid-bearing monzodiorite foid-bearing monzogabbro',
        '10\': foid-bearing diorite foid-bearing gabbro foid-bearing anorthosite',
        '11: foid syenite',
        '12: foid monzosyenite',
        '13: foid monzodiorite foid monzogabbro',
        '14: foid diorite foid gabbro',
        '15: foidolite'
    ]

    LocationAreas1 = [
        [[30.0, 51.96152422706631], [10.0, 17.32050807568877], [38.0, 17.32050807568877], [44.0, 51.96152422706631]],
        [[44.0, 51.96152422706631], [38.0, 17.32050807568877], [62.0, 17.32050807568877], [56.0, 51.96152422706631]],
        [[56.0, 51.96152422706631], [62.0, 17.32050807568877], [90.0, 17.32050807568877], [70.0, 51.96152422706631]],
        [[10.0, 17.32050807568877], [2.5, 4.330127018922193], [12.0, 4.330127018922193], [18.0, 17.32050807568877]],
        [[18.0, 17.32050807568877], [12.0, 4.330127018922193], [35.75, 4.330127018922193], [38.0, 17.32050807568877]],
        [[38.0, 17.32050807568877], [35.75, 4.330127018922193], [64.25, 4.330127018922193], [62.0, 17.32050807568877]],
        [[62.0, 17.32050807568877], [65.0, 0.0], [100.0, 0.0], [90.0, 17.32050807568877]],
        [[2.5, 4.330127018922193], [0.0, 0.0], [10.0, 0.0], [11.650485436893202, 4.204006814487566]],
        [[11.650485436893202, 4.204006814487566], [10.0, 0.0], [35.0, 0.0], [35.75, 4.330127018922193]],
        [[35.75, 4.330127018922193], [35.0, 0.0], [65.0, 0.0], [64.25, 4.330127018922193]],
        [[0.0, 0.0], [5.0, -8.660254037844386], [14.0, -8.660254037844386], [10.0, 0.0]],
        [[10.0, 0.0], [14.0, -8.660254037844386], [36.42857142857143, -8.247860988423225], [35.0, 0.0]],
        [[35.0, 0.0], [36.42857142857143, -8.247860988423225], [63.57142857142858, -8.247860988423225], [65.0, 0.0]],
        [[65.0, 0.0], [63.57142857142858, -8.247860988423225], [95.0, -8.660254037844386], [100.0, 0.0]],
        [[5.0, -8.660254037844386], [30.0, -51.96152422706631], [34.0, -51.96152422706631], [14.0, -8.660254037844386]],
        [[14.0, -8.660254037844386], [34.0, -51.96152422706631], [50.0, -51.96152422706631],
         [50.0, -8.660254037844386]],
        [[50.0, -8.660254037844386], [50.0, -51.96152422706631], [66.0, -51.96152422706631],
         [86.0, -8.660254037844386]],
        [[86.0, -8.660254037844386], [66.0, -51.96152422706631], [70.0, -51.96152422706631],
         [95.0, -8.660254037844386]],
        [[30.0, -51.96152422706631], [45.0, -77.94228634059948], [50.0, -77.94228634059948],
         [50.0, -51.96152422706631]],
        [[50.0, -51.96152422706631], [50.0, -77.94228634059948], [55.0, -77.94228634059948],
         [70.0, -51.96152422706631]],
        [[45.0, -77.94228634059948], [50.0, -86.60254037844386], [55.0, -77.94228634059948]]]

    ItemNames1 = [
        '1:alkali feldspar rhyolite',
        '2:rhyolite',
        '3:dacite',
        '4:quartz alkali feldspar trachyte',
        '5:quartz trachyte',
        '6:quartz latite',
        '7:basalt andesite',
        '8:alkali feldspar trachyte',
        '9:trachyte',
        '10:latite',
        '11:foid-bearing alkali feldspar trachyte',
        '12:foid-bearing trachyte',
        '13:foid-bearing latite',
        '7:basalt andesite',
        '14:phonolite',
        '15:tephritic phonolite',
        '16:phonolitic basanite (olivine > 10%) phonolitic tephrite (olivine < 10%)',
        '17:basanite (olivine > 10%) tephrite (olivine < 10%)',
        '18:phonolitic foidite',
        '19:tephritic foidite',
        '20:foidoite',
    ]

    AreasHeadClosed = []
    SelectDic0 = {}
    SelectDic1 = {}

    AllLabel = []
    IndexList = []
    LabelList = []
    def __init__(self, parent=None, df=pd.DataFrame()):
        QMainWindow.__init__(self, parent)


        self.FileName_Hint='QAPF'
        self._df = df
        self._df_back = df
        if (len(df) > 0):
            self._changed = True
            # print('DataFrame recieved to DualTri')

        self.raw = self._df
        self.create_main_frame()
        self.create_status_bar()

        self.AllLabel = []

        for i in range(len(self._df)):
            tmp_label = self._df.at[i, 'Label']
            if tmp_label not in self.AllLabel:
                self.AllLabel.append(tmp_label)

        for i in range(len(self.LocationAreas0)):
            tmpi = self.LocationAreas0[i] + [self.LocationAreas0[i][0]]
            tmppath = path.Path(tmpi)
            self.AreasHeadClosed.append(tmpi)
            patch = patches.PathPatch(tmppath, facecolor='orange', lw=0.3, alpha=0.3)
            self.SelectDic0[self.ItemNames0[i]] = tmppath

        for i in range(len(self.LocationAreas1)):
            tmpi = self.LocationAreas1[i] + [self.LocationAreas1[i][0]]
            tmppath = path.Path(tmpi)
            self.AreasHeadClosed.append(tmpi)
            patch = patches.PathPatch(tmppath, facecolor='orange', lw=0.3, alpha=0.3)
            self.SelectDic1[self.ItemNames1[i]] = tmppath


        TriLine(Points=[(100, 0, 0), (0, 0, 100), (0, 100, 0), (0, 0, -100), (100, 0, 0), (35, 65, 0)], Sort='',
                Width=1, Color='black', Style='-',
                Alpha=0.7, Label='')

        for i in range(len(self.LabelPosition)):
            plt.annotate(self.Label[i], xy=(self.LabelPosition[i]), xycoords='data', xytext=(0, 0),
                         textcoords='offset points',
                         fontsize=16, )

    def create_main_frame(self):
        self.resize(800, 1000)
        self.main_frame = QWidget()
        self.dpi = 128
        self.fig = Figure((12, 11), dpi=self.dpi)

        self.fig.subplots_adjust(hspace=0.5, wspace=0.5, left=0.1, bottom=0.2, right=0.7, top=0.9)

        # 8 * np.sqrt(3)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)

        self.axes = self.fig.add_subplot(111)
        self.axes.axis('off')
        self.axes.set_xlim(-10, 110)
        self.axes.set_ylim(-105 * np.sqrt(3) / 2, 105 * np.sqrt(3) / 2)
        # Create the navigation toolbar, tied to the canvas
        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)

        # Other GUI controls
        self.save_button = QPushButton('&Save')
        self.save_button.clicked.connect(self.saveImgFile)

        self.result_button = QPushButton('&Classification Result')
        self.result_button.clicked.connect(self.Explain)

        self.legend_cb = QCheckBox('&Legend')
        self.legend_cb.setChecked(True)
        self.legend_cb.stateChanged.connect(self.QAPF)  # int

        self.slider_left_label = QLabel('Plutonic')
        self.slider_right_label = QLabel('Volcanic')


        self.slider = QSlider(Qt.Horizontal)
        self.slider.setRange(0, 1)
        self.slider.setValue(0)
        self.slider.setTracking(True)
        self.slider.setTickPosition(QSlider.TicksBothSides)
        self.slider.valueChanged.connect(self.QAPF)  # int


        self.detail_cb = QCheckBox('&Detail')
        self.detail_cb.setChecked(True)
        self.detail_cb.stateChanged.connect(self.QAPF)  # int



        self.show_data_index_cb = QCheckBox('&Show Data Index')
        self.show_data_index_cb.setChecked(False)
        self.show_data_index_cb.stateChanged.connect(self.QAPF)  # int

        #
        # Layout with box sizers
        #
        self.hbox = QHBoxLayout()

        for w in [self.save_button, self.result_button, self.show_data_index_cb , self.detail_cb, self.legend_cb,self.slider_left_label,self.slider,self.slider_right_label]:
            self.hbox.addWidget(w)
            self.hbox.setAlignment(w, Qt.AlignVCenter)

        self.vbox = QVBoxLayout()
        self.vbox.addWidget(self.mpl_toolbar)
        self.vbox.addWidget(self.canvas)
        self.vbox.addLayout(self.hbox)



        self.textbox = GrowingTextEdit(self)

        self.vbox.addWidget(self.textbox)

        self.main_frame.setLayout(self.vbox)
        self.setCentralWidget(self.main_frame)



        w=self.width()
        h=self.height()

        #setFixedWidth(w/10)

        self.slider.setMinimumWidth(w/10)
        self.slider_left_label.setMinimumWidth(w/10)
        self.slider_right_label.setMinimumWidth(w/10)


    def QAPF(self):

        self.axes.clear()
        self.axes.axis('off')
        self.Tags = []

        raw = self._df

        self.axes.set_xlim(-10, 110)
        self.axes.set_ylim(-105 * np.sqrt(3) / 2, 105 * np.sqrt(3) / 2)



        if (int(self.slider.value()) == 0):
            s = [
                TriLine(Points=[(100, 0, 0), (0, 0, 100), (0, 100, 0), (0, 0, -100), (100, 0, 0), (0, 100, 0)], Sort='',
                        Width=1, Color='black', Style='-',
                        Alpha=0.7, Label='')]
            for i in s:
                self.axes.plot(i.X, i.Y, color=i.Color, linewidth=i.Width, linestyle=i.Style, alpha=i.Alpha,
                               label=i.Label)
            self.Labels = ['1a',

                           '1b',

                           '2',
                           '3a',
                           '3b',
                           '4',
                           '5',

                           '6*',
                           '7*',
                           '8*',
                           '9*',
                           '10*',

                           '6',
                           '7',
                           '8',
                           '9',
                           '10',

                           '6\'',
                           '7\'',
                           '8\'',
                           '9\'',
                           '10\'',

                           '11',
                           '12',
                           '13',
                           '14',
                           '15']
            self.Locations = [(50,80),

                              (50,65),

                              (22,33),
                              (32,33),
                              (50,33),
                              (66,33),
                              (76,33),

                              (10, 10),
                              (26, 10),
                              (50, 10),
                              (74, 10),
                              (88, 10),

                              (6, 1),
                              (24, 1),
                              (50, 1),
                              (76, 1),
                              (90, 1),

                              (6, -5),
                              (24, -5),
                              (50, -5),
                              (76, -5),
                              (90, -5),

                              (18, -30),
                              (40, -30),
                              (60, -30),
                              (78, -30),
                              (50, -60)]


            self.setWindowTitle('QAPF modal classification of plutonic rocks')

            self.exptext='''
                1a: quartzolite,
                1b: quartz-rich granitoid,		
                2: alkali feldspar granite,
                3a: (syeno granite),
                3b: (monzo granite),
                4: granodiorite,
                5: tonalite,		
                6*: quartz alkali feldspar syenite,
                7*: quartz syenite,
                8*: quartz monzonite,
                9*: quartz monzodiorite quartz monzogabbro,
                10*: quartz diorite quartz gabbro  quartz anorthosite,		
                6: alkali feldspar syenite,
                7: syenite,
                8: monzonite,
                9: monzodiorite monzogabbro,
                10: diorite gabbro anorthosite,		
                6\': foid-bearing alkali feldspar syenite,
                7\': foid-bearing syenite,
                8\': foid-bearing monzonite,
                9\': foid-bearing monzodiorite foid-bearing monzogabbro,
                10\': foid-bearing diorite foid-bearing gabbro foid-bearing anorthosite,		
                11: foid syenite,
                12: foid monzosyenite,
                13: foid monzodiorite foid monzogabbro,
                14: foid diorite foid gabbro,
                15: foidolite
                '''

            self.textbox.setText(self.reference + '\n' + self.infotext+ '\n' + self.exptext)


            D1 = (0, 0, 100)
            L1 = [(10, 0, 90), (0, 10, 90)]
            L2 = [(40, 0, 60), (0, 40, 60)]
            L3 = [(80, 0, 20), (0, 80, 20)]

            L4 = [(95, 0, 5), (0, 95, 5)]

            SL1 = [D1, (90, 10, 0)]
            SL2 = [D1, (65, 35, 0)]
            SL3 = [D1, (35, 65, 0)]
            SL4 = [D1, (10, 90, 0)]

            CL1 = self.TriCross(SL1, L2)
            CL21 = self.TriCross(SL2, L2)
            CL22 = self.TriCross(SL2, L3)
            CL3 = self.TriCross(SL3, L2)
            CL41 = self.TriCross(SL4, L2)
            CL42 = self.TriCross(SL4, L3)

            NSL1 = [CL1, (90, 10, 0)]
            NSL21 = [CL21, CL22]
            NSL22 = [CL22, (65, 35, 0)]
            NSL3 = [CL3, (35, 65, 0)]
            NSL4 = [CL41, (10, 90, 0)]

            s = [TriLine(Points=L1, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),
                 TriLine(Points=L2, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),
                 TriLine(Points=L3, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),
                 TriLine(Points=L4, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),
                 TriLine(Points=NSL1, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),
                 TriLine(Points=NSL21, Sort='', Width=1, Color='black', Style='--', Alpha=0.7,
                         Label=''),
                 TriLine(Points=NSL22, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),
                 TriLine(Points=NSL3, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),
                 TriLine(Points=NSL4, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label='')]

            for i in s:
                self.axes.plot(i.X, i.Y, color=i.Color, linewidth=i.Width, linestyle=i.Style, alpha=i.Alpha,
                               label=i.Label)

            D2 = (0, 0, -100)
            L2 = [(40, 0, -60), (0, 40, -60)]
            L3 = [(90, 0, -10), (0, 90, -10)]

            SL1 = [D2, (90, 10, 0)]
            SL2 = [D2, (65, 35, 0)]
            SL3 = [D2, (35, 65, 0)]
            SL4 = [D2, (10, 90, 0)]

            SL5 = [(20, 20, -60), (45, 45, -10)]

            CL1 = self.TriCross(SL1, L2)
            CL2 = self.TriCross(SL2, L3)
            CL3 = self.TriCross(SL3, L3)
            CL41 = self.TriCross(SL4, L2)
            CL42 = self.TriCross(SL4, L3)

            NSL1 = [CL1, (90, 10, 0)]
            NSL2 = [CL2, (65, 35, 0)]
            NSL3 = [CL3, (35, 65, 0)]
            NSL4 = [CL41, (10, 90, 0)]

            s = [
                TriLine(Points=L2, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                        Label=''),
                TriLine(Points=L3, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                        Label=''),
                TriLine(Points=SL5, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                        Label=''),
                TriLine(Points=NSL1, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                        Label=''),
                TriLine(Points=NSL2, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                        Label=''),
                TriLine(Points=NSL3, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                        Label=''),
                TriLine(Points=NSL4, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                        Label='')]
            for i in s:
                self.axes.plot(i.X, i.Y, color=i.Color, linewidth=i.Width, linestyle=i.Style, alpha=i.Alpha,
                               label=i.Label)


        else:

            s = [
                TriLine(Points=[ (35, 65, 0),(100, 0, 0), (0, 0, 100), (0, 100, 0),(0, 0, -100), (100, 0, 0)], Sort='',
                        Width=1, Color='black', Style='-',
                        Alpha=0.7, Label='')]
            for i in s:
                self.axes.plot(i.X, i.Y, color=i.Color, linewidth=i.Width, linestyle=i.Style, alpha=i.Alpha,
                               label=i.Label)


            self.Labels = ['1',
                           '2',
                           '3',

                           '4',
                           '5',
                           '6',
                           '7',

                           '8',
                           '9',
                           '10',

                           '11',
                           '12',
                           '13',

                           '14',
                           '15',
                           '16',
                           '17',

                           '18',
                           '19',
                           '20']

            self.Locations = [(22,33),
                              (42,33),
                              (65,33),

                              (10, 10),
                              (25, 10),
                              (50, 10),
                              (80, 10),

                              (6, 1),
                              (24, 1),
                              (50, 1),

                              (6, -5),
                              (24, -5),
                              (50, -5),

                              (18, -30),
                              (40, -30),
                              (60, -30),
                              (78, -30),


                              (40, -63),
                              (55, -63),
                              (48, -82)]


            self.setWindowTitle('QAPF modal classification of volcanic rocks')



            self.exptext='''
                1:alkali feldspar rhyolite,
                2:rhyolite,
                3:dacite,
                4:quartz alkali feldspar trachyte,
                5:quartz trachyte,
                6:quartz latite,
                7:basalt andesite,
                8:alkali feldspar trachyte,
                9:trachyte,
                10:latite,
                11:foid-bearing alkali feldspar trachyte,
                12:foid-bearing trachyte,
                13:foid-bearing latite,
                14:phonolite,
                15:tephritic phonolite,
                16:phonolitic basanite (olivine > 10%) phonolitic tephrite (olivine < 10%),
                17:basanite (olivine > 10%) tephrite (olivine < 10%),
                18:phonolitic foidite,
                19:tephritic foidite,
                20:foidoite,         
                '''

            self.textbox.setText(self.reference + '\n' + self.infotext+ '\n' + self.exptext)


            D = (0, 0, 100)
            L1 = [(10, 0, 90), (0, 10, 90)]
            L2 = [(40, 0, 60), (0, 40, 60)]
            L3 = [(80, 0, 20), (0, 80, 20)]

            L4 = [(95, 0, 5), (0, 95, 5)]

            s = [TriLine(Points=L1, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),

                 TriLine(Points=L2, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),

                 TriLine(Points=L3, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''), ]

            for i in s:
                self.axes.plot(i.X, i.Y, color=i.Color, linewidth=i.Width, linestyle=i.Style, alpha=i.Alpha,
                               label=i.Label)

            SL1 = [D, (90, 10, 0)]
            SL2 = [D, (65, 35, 0)]
            SL3 = [D, (35, 65, 0)]
            SL4 = [D, (10, 90, 0)]

            CL1 = self.TriCross(SL1, L2)
            CL21 = self.TriCross(SL2, L2)
            CL22 = self.TriCross(SL2, L3)
            CL3 = self.TriCross(SL3, L2)
            CL41 = self.TriCross(SL4, L2)
            CL42 = self.TriCross(SL4, L3)

            TL4 = self.TriCross(SL3, L4)

            NL4 = [(95, 0, 5), TL4]

            NSL1 = [CL1, (90, 10, 0)]
            NSL21 = [CL21, CL22]
            NSL22 = [CL22, (65, 35, 0)]
            NSL3 = [CL3, (35, 65, 0)]
            NSL4 = [CL41, CL42]

            s = [TriLine(Points=NL4, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),

                 TriLine(Points=NSL1, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),

                 TriLine(Points=NSL21, Sort='', Width=1, Color='black', Style='--', Alpha=0.7,
                         Label=''),

                 TriLine(Points=NSL22, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),

                 TriLine(Points=NSL3, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),

                 TriLine(Points=NSL4, Sort='', Width=1, Color='black', Style='--', Alpha=0.7,
                         Label='')]

            for i in s:
                self.axes.plot(i.X, i.Y, color=i.Color, linewidth=i.Width, linestyle=i.Style, alpha=i.Alpha,
                               label=i.Label)

            D = (0, 0, -100)
            L1 = [(10, 0, -90), (0, 10, -90)]
            L2 = [(40, 0, -60), (0, 40, -60)]
            L3 = [(90, 0, -10), (0, 90, -10)]

            SL5 = [(5, 5, -90), (45, 45, -10)]

            s = [TriLine(Points=L1, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),
                 TriLine(Points=L2, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),
                 TriLine(Points=L3, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),
                 TriLine(Points=SL5, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label='')]

            for i in s:
                self.axes.plot(i.X, i.Y, color=i.Color, linewidth=i.Width, linestyle=i.Style, alpha=i.Alpha,
                               label=i.Label)

            SL1 = [D, (90, 10, 0)]
            SL2 = [D, (65, 35, 0)]
            SL3 = [D, (35, 65, 0)]
            SL4 = [D, (10, 90, 0)]

            CL1 = self.TriCross(SL1, L2)
            CL2 = self.TriCross(SL2, L3)
            CL3 = self.TriCross(SL3, L3)
            CL41 = self.TriCross(SL4, L2)
            CL42 = self.TriCross(SL4, L3)

            NSL1 = [CL1, (90, 10, 0)]
            NSL2 = [CL2, (65, 35, 0)]
            NSL3 = [CL3, (35, 65, 0)]
            NSL4 = [CL41, CL42]

            s = [TriLine(Points=NSL1, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),

                 TriLine(Points=NSL2, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),

                 TriLine(Points=NSL3, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label=''),

                 TriLine(Points=NSL4, Sort='', Width=1, Color='black', Style='-', Alpha=0.7,
                         Label='')]

            for i in s:
                self.axes.plot(i.X, i.Y, color=i.Color, linewidth=i.Width, linestyle=i.Style, alpha=i.Alpha,
                               label=i.Label)

        for i in range(len(self.LabelPosition)):
            self.axes.annotate(self.Label[i], xy=(self.LabelPosition[i]), xycoords='data', xytext=(0, 0),
                               textcoords='offset points',
                               fontsize=8, )




        for i in range(len(self.Labels)):
            self.Tags.append(Tag(Label=self.Labels[i],
                                 Location=(self.Locations[i][0], self.Locations[i][1]),
                                 ))

        if (self.detail_cb.isChecked()):
            for i in self.Tags:
                self.axes.annotate(i.Label, xy=i.Location,
                                   fontsize=6, color='grey', alpha=0.8)



        #raw = self.CleanDataFile(self._df)


        PointLabels = []
        TPoints = []
        self.IndexList = []
        self.LabelList = []
        self.TypeList = []

        print(raw.columns.values)

        for i in range(len(raw)):

            if 'A (Volume)' in raw.columns.values:
                q = raw.at[i, 'Q (Volume)']
                f = raw.at[i, 'F (Volume)']
                a = raw.at[i, 'A (Volume)']
                p = raw.at[i, 'P (Volume)']
            elif 'A' in raw.columns.values:
                q = raw.at[i, 'Q']
                f = raw.at[i, 'F']
                a = raw.at[i, 'A']
                p = raw.at[i, 'P']


            TmpLabel = ''
            if (raw.at[i, 'Label'] in PointLabels or raw.at[i, 'Label'] == ''):
                TmpLabel = ''
            else:
                PointLabels.append(raw.at[i, 'Label'])
                TmpLabel = raw.at[i, 'Label']

            self.LabelList.append(raw.at[i, 'Label'])
            if 'Index' in raw.columns.values:
                self.IndexList.append(raw.at[i, 'Index'])
            else:
                self.IndexList.append('No ' + str(int(i+1)))



            if (q != 0 and q != ''):
                TPoints.append(TriPoint((a, p, q), Size=raw.at[i, 'Size'],
                                        Color=raw.at[i, 'Color'], Alpha=raw.at[i, 'Alpha'], Marker=raw.at[i, 'Marker'],
                                        Label=TmpLabel))


                xa,ya = self.TriToBin(a, p, q)

            else:
                TPoints.append(TriPoint((a, p, 0 - f), Size=raw.at[i, 'Size'],
                                        Color=raw.at[i, 'Color'], Alpha=raw.at[i, 'Alpha'], Marker=raw.at[i, 'Marker'],
                                        Label=TmpLabel))



                xa,ya = self.TriToBin(a, p, 0 - f)



            if (int(self.slider.value()) == 0):
                HitOnRegions = 0
                for j in self.ItemNames0:
                    if self.SelectDic0[j].contains_point([xa, ya]):
                        self.TypeList.append(j)
                        HitOnRegions = 1
                        break
                if HitOnRegions == 0:
                    self.TypeList.append('on line or out')
            else:
                HitOnRegions = 0
                for j in self.ItemNames1:
                    if self.SelectDic1[j].contains_point([xa, ya]):
                        self.TypeList.append(j)
                        HitOnRegions = 1
                        break
                if HitOnRegions == 0:
                    self.TypeList.append('on line or out')




        for i in range(len(TPoints)):
            self.axes.scatter(TPoints[i].X, TPoints[i].Y, marker=TPoints[i].Marker, s=TPoints[i].Size, color=TPoints[i].Color, alpha=TPoints[i].Alpha,
                              label=TPoints[i].Label)


            if (self.show_data_index_cb.isChecked()):

                if 'Index' in self._df_back.columns.values:

                    self.axes.annotate(str(self._df_back.at[i, 'Index']),
                                       xy=(TPoints[i].X,
                                           TPoints[i].Y),
                                       color=self._df.at[i, 'Color'],
                                       alpha=self._df.at[i, 'Alpha'])
                else:
                    self.axes.annotate('No' + str(i + 1),
                                       xy=(TPoints[i].X,
                                           TPoints[i].Y),
                                       color=self._df.at[i, 'Color'],
                                       alpha=self._df.at[i, 'Alpha'])




        if (self.legend_cb.isChecked()):
            self.axes.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0, prop=fontprop)

        self.canvas.draw()

        #self.OutPutFig=self.fig

        self.OutPutTitle = 'QAPF'

        print(len(self.LabelList), len(self.IndexList), len(self.TypeList))

        self.OutPutData = pd.DataFrame({'Label': self.LabelList,
                                        'Index': self.IndexList,
                                        'Type': self.TypeList,
                                        })

    def Explain(self):

        # self.OutPutData = self.OutPutData.set_index('Label')

        self.tablepop = TableViewer(df=self.OutPutData, title='QPAF Result')
        self.tablepop.show()


from geopytool.ImportDependence import *
from geopytool.CustomClass import *
#from geopytool.TableViewer import TableViewer



class ZrYSrTi(AppForm):
    _df = pd.DataFrame()
    _changed = False

    title= 'Zr, Ti, Y and Sr analyses of oceanic basalts'

    xlabel = r'ld1'
    ylabel = r'ld2'

    itemstocheck = ['Zr', 'Y', 'Sr','Ti']
    reference = 'Vermeesch, P. 2006, Tectonic discrimination diagrams revisited,Geochem. Geophys. Geosyst.,7, Q06017,doi:10.1029/2005GC001092\nButler, R., and A. Woronow (1986), Discrimination among tectonic settings using trace element abundances of basalts, J. Geophys. Res., 91, 10,289–10,300.'

    ItemNames = ['IAB',
                 'MORB',
                 'OIB',]

    '''
    [[-12, 4], [-12.23, -1.37], [-18, -6.6], [-8, -6.45],],
    [[5.02, -6.28], [12.17, -12.23], [15.9, -10.93], [11.85, -16],],

    '''

    LocationAreas = [ [[5.02,-6.28], [12.17, -12.23], [11.85, -16],[5.02,-16]],
                    [[5.02, -6.28], [12.17, -12.23], [15.9, -10.93], [15.9, -5],[5.02,-5]],
                    [[15.9, -10.93], [12.17, -12.23],[11.85, -16],[15.9,-16]],
                    ]

    AreasHeadClosed = []
    SelectDic = {}
    LabelList = []
    IndexList = []
    TypeList=[]


    All_X = []
    All_Y = []

    whole_labels=[]
    SVM_labels=[]

    Ti_TiO2_ratio_mass = 47.867 / (47.867 + 32)


    def create_main_frame(self):
        self.resize(1000, 800)
        self.main_frame = QWidget()
        self.dpi = 128
        self.fig = Figure((18.0, 12.0), dpi=self.dpi)
        self.fig.subplots_adjust(hspace=0.5, wspace=0.5, left=0.1, bottom=0.2, right=0.7, top=0.9)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)
        self.axes = self.fig.add_subplot(111)
        self.axes.axis('off')
        # Create the navigation toolbar, tied to the canvas
        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)

        # Other GUI controls


        self.load_data_button = QPushButton('&Add Data to Compare')
        self.load_data_button.clicked.connect(self.loadDataToTest)


        self.save_button = QPushButton('&Save Img')
        self.save_button.clicked.connect(self.saveImgFile)

        self.result_button = QPushButton('&Classification Result')
        self.result_button.clicked.connect(self.Explain)


        self.save_predict_button_selected = QPushButton('&Predict Result')
        self.save_predict_button_selected.clicked.connect(self.showPredictResultSelected)

        self.legend_cb = QCheckBox('&Legend')
        self.legend_cb.setChecked(True)
        self.legend_cb.stateChanged.connect(self.Key_Func)  # int


        self.show_load_data_cb = QCheckBox('&Show Loaded Data')
        self.show_load_data_cb.setChecked(True)
        self.show_load_data_cb.stateChanged.connect(self.Key_Func)  # int

        self.show_data_index_cb = QCheckBox('&Show Data Index')
        self.show_data_index_cb.setChecked(False)
        self.show_data_index_cb.stateChanged.connect(self.Key_Func)  # int


        self.detail_cb = QCheckBox('&Detail')
        self.detail_cb.setChecked(True)
        self.detail_cb.stateChanged.connect(self.Key_Func)  # int

        self.shape_cb= QCheckBox('&Shape')
        self.shape_cb.setChecked(False)
        self.shape_cb.stateChanged.connect(self.Key_Func)  # int


        self.hyperplane_cb= QCheckBox('&SVM Boundary')
        self.hyperplane_cb.setChecked(False)
        self.hyperplane_cb.stateChanged.connect(self.Key_Func)  # int


        self.kernel_select = QSlider(Qt.Horizontal)
        self.kernel_select.setRange(0, len(self.kernel_list)-1)
        self.kernel_select.setValue(0)
        self.kernel_select.setTracking(True)
        self.kernel_select.setTickPosition(QSlider.TicksBothSides)
        self.kernel_select.valueChanged.connect(self.Key_Func)  # int
        self.kernel_select_label = QLabel('Kernel')

        #
        # Layout with box sizers
        #
        self.hbox = QHBoxLayout()
        self.hbox1 = QHBoxLayout()
        self.hbox2 = QHBoxLayout()

        for w in [self.load_data_button,self.save_button, self.result_button, self.save_predict_button_selected,]:
            self.hbox1.addWidget(w)
            self.hbox1.setAlignment(w, Qt.AlignVCenter)

        for w in [self.legend_cb,self.show_load_data_cb,self.show_data_index_cb,self.shape_cb,self.hyperplane_cb,self.kernel_select_label,self.kernel_select ,self.detail_cb,]:
            self.hbox2.addWidget(w)
            self.hbox2.setAlignment(w, Qt.AlignVCenter)



        self.vbox = QVBoxLayout()
        self.vbox.addWidget(self.mpl_toolbar)
        self.vbox.addWidget(self.canvas)
        self.vbox.addLayout(self.hbox1)
        self.vbox.addLayout(self.hbox2)
        self.textbox = GrowingTextEdit(self)
        self.textbox.setText(self.reference+    '''
            Use natural logarithms instead of base 10\n
            ld1= -0.016*np.log(Zr/Ti) - 2.961*np.log(Y/Ti) + 1.500*np.log(Sr/Ti)\n    
            ld2= -1.474*np.log(Zr/Ti) + 2.143*np.log(Y/Ti) + 1.840*np.log(Sr/Ti)
            ''')

        self.vbox.addWidget(self.textbox)

        self.main_frame.setLayout(self.vbox)
        self.setCentralWidget(self.main_frame)

        w=self.width()
        h=self.height() 

    def loadDataToTest(self):
        TMP =self.getDataFile()
        if TMP != 'Blank':
            self.data_to_test=TMP[0]
        self.Key_Func()


    '''
    Use natural logarithms instead of base 10\n
    ld1= -0.016*np.log(Zr/Ti) - 2.961*np.log(Y/Ti) + 1.500*np.log(Sr/Ti)\n    
    ld2= -1.474*np.log(Zr/Ti) + 2.143*np.log(Y/Ti) + 1.840*np.log(Sr/Ti)
    '''
    

    def ld1(self,Zr=1,Y=1,Sr=1,Ti=1):
        return(-0.016*np.log(Zr/Ti) - 2.961*np.log(Y/Ti) + 1.500*np.log(Sr/Ti))

    def ld2(self,Zr=1,Y=1,Sr=1,Ti=1):
        return(-1.474*np.log(Zr/Ti) + 2.143*np.log(Y/Ti) + 1.840*np.log(Sr/Ti))

    def Key_Func(self, height=12, dpi=300):
        k_s = int(self.kernel_select.value())
        self.kernel_select_label.setText(self.kernel_list[k_s])
        self.setWindowTitle(self.title)
        self.axes.clear()
        #self.axes.axis('off')
        self.FileName_Hint= self.title        
        self.axes.set_xlabel(self.xlabel)
        self.axes.set_ylabel(self.ylabel)
        #self.axes.spines['right'].set_color('none')
        #self.axes.spines['top'].set_color('none')

        all_labels=[]
        all_colors=[]
        all_markers=[]
        all_alpha=[]

        self.OutPutData = pd.DataFrame()

        self.LabelList=[]
        self.IndexList=[]
        self.TypeList=[]




        for i in range(len(self._df)):
            target = self._df.at[i, 'Label']
            color = self._df.at[i, 'Color']
            marker = self._df.at[i, 'Marker']
            alpha = self._df.at[i, 'Alpha']

            if target not in all_labels:
                all_labels.append(target)
                all_colors.append(color)
                all_markers.append(marker)
                all_alpha.append(alpha)

        self.whole_labels = all_labels

        PointLabels = []
        PointColors = []
        x = []
        y = []
        
        Zr = []
        Ti = []
        Y = []
        Sr = []
        TiO2 = []

        ItemNames = ['IAB',
                'MORB',
                'OIB',]


        Locations = [(10, -8), (8, -14), (14, -14)]
        X_offset = -6
        Y_offset = 3

        TagNumber = min(len(self.ItemNames), len(Locations))
        if (self.detail_cb.isChecked()):
            for k in range(TagNumber):
                self.axes.annotate(self.ItemNames[k], Locations[k], xycoords='data', xytext=(X_offset, Y_offset),
                                   textcoords='offset points',
                                   fontsize=9, color='grey', alpha=0.8)

        self.DrawLine([(5.02, -6.28), (12.17, -12.23)])
        self.DrawLine([(12.17, -12.23), (11.85, -16)])
        self.DrawLine([(12.17, -12.23), (15.9, -10.93)])


        if (self._changed):
            df =  self.CleanDataFile(self._df)

            
            XtoFit = {}
            YtoFit = {}

            SVM_X=[]
            SVM_Y=[]

            for i in range(len(df)):
                TmpLabel = ''
                if (df.at[i, 'Label'] not in PointLabels or df.at[i, 'Label'] == ''):
                    PointLabels.append(df.at[i, 'Label'])

            for i in  PointLabels:
                XtoFit[i]=[]
                YtoFit[i]=[]

            TmpPointLabels=[]

            for i in range(len(df)):
                Zr_tmp = (df.at[i, 'Zr'])
                Y_tmp = (df.at[i, 'Y'])
                Sr_tmp = (df.at[i, 'Sr'])

                if 'TiO2' in df.columns.values.tolist():
                    tmp_data = df.at[i, 'TiO2']*self.Ti_TiO2_ratio_mass*10000
                    Ti_tmp = tmp_data
                elif 'Ti' in df.columns.values.tolist():
                    Ti_tmp = df.at[i, 'Ti']
                else:
                    Ti_tmp=0

                if Ti_tmp !=0:
                    pass
                    Zr.append(Zr_tmp)
                    Y.append(Y_tmp)
                    Sr.append(Sr_tmp)
                    Ti.append(Ti_tmp)

                    xtest= self.ld1(Zr_tmp,Y_tmp,Sr_tmp,Ti_tmp)
                    ytest= self.ld2(Zr_tmp,Y_tmp,Sr_tmp,Ti_tmp)


                    Alpha = df.at[i, 'Alpha']
                    Marker = df.at[i, 'Marker']
                    Label = df.at[i, 'Label']

                    TmpLabel = ''
                    if (df.at[i, 'Label'] in TmpPointLabels or df.at[i, 'Label'] == ''):
                        TmpLabel = ''
                    else:
                        TmpPointLabels.append(df.at[i, 'Label'])
                        TmpLabel = df.at[i, 'Label']

                    TmpColor = ''
                    if (df.at[i, 'Color'] in PointColors or df.at[i, 'Color'] == ''):
                        TmpColor = ''
                    else:
                        PointColors.append(df.at[i, 'Color'])
                        TmpColor = df.at[i, 'Color']

                    XtoFit[Label].append(xtest)
                    YtoFit[Label].append(ytest)

                    SVM_X.append(xtest)
                    SVM_Y.append(ytest)

                    HitOnRegions = 0

                    self.LabelList.append(Label)

                    if 'Index' in self._df_back.columns.values:
                        self.IndexList.append(self._df_back.at[i, 'Index'])
                    else:
                        self.IndexList.append('No ' + str(int(i+1)))


                    for j in self.ItemNames:
                        if self.SelectDic[j].contains_point([xtest,ytest]):
                            HitOnRegions = 1
                            self.TypeList.append(j)
                            break
                    if HitOnRegions == 0:
                        self.TypeList.append('on line or out')

                    self.axes.scatter(xtest, ytest, marker=df.at[i, 'Marker'],
                      s=df.at[i, 'Size'], color=df.at[i, 'Color'], alpha=df.at[i, 'Alpha'], label=TmpLabel)
                      #edgecolors='black')





            if (self.shape_cb.isChecked()):
                for i in PointLabels:

                    if XtoFit[i] != YtoFit[i]:
                        xmin, xmax = min(XtoFit[i]), max(XtoFit[i])
                        ymin, ymax = min(YtoFit[i]), max(YtoFit[i])

                        DensityColorMap = 'Greys'
                        DensityAlpha = 0.1

                        DensityLineColor = PointColors[PointLabels.index(i)]
                        DensityLineAlpha = 0.3

                        # Peform the kernel density estimate
                        xx, yy = np.mgrid[xmin:xmax:200j, ymin:ymax:200j]
                        # print(self.ShapeGroups)
                        # command='''xx, yy = np.mgrid[xmin:xmax:'''+str(self.ShapeGroups)+ '''j, ymin:ymax:''' +str(self.ShapeGroups)+'''j]'''
                        # exec(command)
                        # print(xx, yy)
                        positions = np.vstack([xx.ravel(), yy.ravel()])
                        values = np.vstack([XtoFit[i], YtoFit[i]])
                        kernelstatus = True
                        try:
                            st.gaussian_kde(values)
                        except Exception as e:
                            self.ErrorEvent(text=repr(e))
                            kernelstatus = False
                        if kernelstatus == True:
                            kernel = st.gaussian_kde(values)
                            f = np.reshape(kernel(positions).T, xx.shape)
                            # Contourf plot
                            cfset = self.axes.contourf(xx, yy, f, cmap=DensityColorMap, alpha=DensityAlpha)
                            ## Or kernel density estimate plot instead of the contourf plot
                            # self.axes.imshow(np.rot90(f), cmap='Blues', extent=[xmin, xmax, ymin, ymax])
                            # Contour plot
                            cset = self.axes.contour(xx, yy, f, colors=DensityLineColor, alpha=DensityLineAlpha)
                            # Label plot
                            #self.axes.clabel(cset, inline=1, fontsize=10)


            if (len(self.data_to_test) > 0):
                self.data_to_test_back= self.data_to_test
                contained = True
                missing = 'Miss setting infor:'

                for i in ['Label', 'Color', 'Marker', 'Alpha']:
                    if i not in self.data_to_test.columns.values.tolist():
                        contained = False
                        missing = missing + '\n' + i

                if contained == True:
                    for i in self.data_to_test.columns.values.tolist():
                        if i not in self._df.columns.values.tolist():
                            self.data_to_test = self.data_to_test.drop(columns=i)

                    # print(self.data_to_test)

                    test_labels = []
                    test_colors = []
                    test_markers = []
                    test_alpha = []

                    for i in range(len(self.data_to_test)):

                        # print(self.data_to_test.at[i, 'Label'])
                        target = self.data_to_test.at[i, 'Label']
                        color = self.data_to_test.at[i, 'Color']
                        marker = self.data_to_test.at[i, 'Marker']
                        alpha = self.data_to_test.at[i, 'Alpha']

                        if target not in test_labels and target not in all_labels:
                            test_labels.append(target)
                            test_colors.append(color)
                            test_markers.append(marker)
                            test_alpha.append(alpha)

                    self.whole_labels = self.whole_labels + test_labels


                    self.load_settings_backup = self.data_to_test
                    Load_ItemsToTest = ['Label', 'Number', 'Tag', 'Name', 'Author', 'DataType', 'Marker', 'Color',
                                        'Size',
                                        'Alpha',
                                        'Style', 'Width']
                    for i in self.data_to_test.columns.values.tolist():
                        if i not in Load_ItemsToTest:
                            self.load_settings_backup = self.load_settings_backup.drop(i, 1)

                    print(self.load_settings_backup, self.data_to_test)

                    print(self.load_settings_backup.shape, self.data_to_test.shape)


                    try:

                        for i in range(len(self.data_to_test)):

                            target = self.data_to_test.at[i, 'Label']
                            if target not in all_labels:
                                all_labels.append(target)
                                tmp_label = self.data_to_test.at[i, 'Label']
                            else:
                                tmp_label=''


                            Zr_tmp_to_test = (self.data_to_test.at[i, 'Zr'])
                            Y_tmp_to_test = (self.data_to_test.at[i, 'Y'])
                            Sr_tmp_to_test = (self.data_to_test.at[i, 'Sr'])

                            if 'TiO2' in self.data_to_test.columns.values.tolist():
                                tmp_data = self.data_to_test.at[i, 'TiO2']*self.Ti_TiO2_ratio_mass*10000
                                Ti_tmp_to_test = tmp_data
                            elif  'Ti' in self.data_to_test.columns.values.tolist():
                                Ti_tmp_to_test = self.data_to_test.at[i, 'Ti']
                            else:
                                Ti_tmp_to_test = 0

                            if Ti_tmp_to_test != 0:
                                x_load_test = self.ld1(Zr_tmp_to_test,Y_tmp_to_test,Sr_tmp_to_test,Ti_tmp_to_test)
                                y_load_test = self.ld2(Zr_tmp_to_test,Y_tmp_to_test,Sr_tmp_to_test,Ti_tmp_to_test)


                                for j in self.ItemNames:
                                    if self.SelectDic[j].contains_point([x_load_test, y_load_test]):
                                        self.LabelList.append(self.data_to_test.at[i, 'Label'])
                                        self.TypeList.append(j)


                                        print(self.data_to_test.at[i, 'Label'],j)

                                        if 'Index' in self.data_to_test_back.columns.values:
                                            self.IndexList.append(self.data_to_test_back.at[i, 'Index'])
                                        else:
                                            self.IndexList.append('No '+str(int(i+1)))
                                        break

                                if (self.show_load_data_cb.isChecked()):

                                    self.axes.scatter(x_load_test,y_load_test,
                                                      marker=self.data_to_test.at[i, 'Marker'],
                                                      s=self.data_to_test.at[i, 'Size'],
                                                      color=self.data_to_test.at[i, 'Color'],
                                                      alpha=self.data_to_test.at[i, 'Alpha'],
                                                      label=tmp_label)
                                                      #edgecolors='black')


                    except Exception as e:
                        self.ErrorEvent(text=repr(e))


            if (self.legend_cb.isChecked()):
                self.axes.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0, prop=fontprop)


            self.All_X=SVM_X
            self.All_Y=SVM_Y

            if (self.hyperplane_cb.isChecked()):
                clf = svm.SVC(C=1.0, kernel=self.kernel_list[k_s], probability=True)

                svm_x= SVM_X
                svm_y= SVM_Y

                print(len(svm_x),len(svm_y),len(df.index))

                xx, yy = np.meshgrid(np.arange(min(svm_x), max(svm_x), np.ptp(svm_x) / 200),
                                     np.arange(min(svm_y), max(svm_y), np.ptp(svm_y) / 200))

                le = LabelEncoder()
                le.fit(self._df.Label)
                class_label = le.transform(self._df.Label)
                svm_train= pd.concat([pd.DataFrame(svm_x),pd.DataFrame(svm_y)], axis=1)
                svm_train=svm_train.values
                clf.fit(svm_train,class_label)
                Z = clf.predict(np.c_[xx.ravel(), yy.ravel()])
                Z = Z.reshape(xx.shape)
                self.axes.contourf(xx, yy, Z, cmap='hot', alpha=0.2)

            if (self.show_data_index_cb.isChecked()):

                if 'Index' in self._df.columns.values:

                    for i in range(len(self._df)):
                        self.axes.annotate(self._df.at[i, 'Index'],
                                           xy=(self.All_X[i],
                                               self.All_Y[i]),
                                           color=self._df.at[i, 'Color'],
                                           alpha=self._df.at[i, 'Alpha'])
                else:
                    for i in range(len(self._df)):
                        self.axes.annotate('No' + str(i + 1),
                                           xy=(self.All_X[i],
                                               self.All_Y[i]),
                                           color=self._df.at[i, 'Color'],
                                           alpha=self._df.at[i, 'Alpha'])


            self.canvas.draw()





        self.OutPutTitle='Key_Func'

        self.OutPutData = pd.DataFrame(
            {'Label': self.LabelList,
             'Index':self.IndexList,
             'Type': self.TypeList,
             })

        self.OutPutFig=self.fig


    def showPredictResultSelected(self):

        k_s = int(self.kernel_select.value())
        try:
            clf = svm.SVC(C=1.0, kernel=self.kernel_list[k_s], probability=True)
            svm_x = self.All_X
            svm_y = self.All_Y
            le = LabelEncoder()
            le.fit(self._df.Label)
            class_label = le.transform(self._df.Label)
            svm_train = pd.concat([pd.DataFrame(svm_x), pd.DataFrame(svm_y)], axis=1)
            svm_train = svm_train.values
            clf.fit(svm_train, self._df.Label)


            xx=[]
            yy=[]


            for i in range(len(self.data_to_test)):

                Zr_tmp = (self.data_to_test.at[i, 'Zr'])
                Y_tmp = (self.data_to_test.at[i, 'Y'])
                Sr_tmp = (self.data_to_test.at[i, 'Sr'])

                if 'TiO2' in self.data_to_test.columns.values.tolist():
                    tmp_data = self.data_to_test.at[i, 'TiO2']*self.Ti_TiO2_ratio_mass*10000
                    Ti_tmp = tmp_data
                elif  'Ti' in self.data_to_test.columns.values.tolist():
                    Ti_tmp = self.data_to_test.at[i, 'Ti']
                else:
                    Ti_tmp =0


                if Ti_tmp != 0:
                    pass
                
                    xtest= self.ld1(Zr_tmp,Y_tmp,Sr_tmp,Ti_tmp)
                    ytest= self.ld2(Zr_tmp,Y_tmp,Sr_tmp,Ti_tmp)

                    xx.append(xtest)
                    yy.append(ytest)

            Z = clf.predict(np.c_[xx.ravel(), yy])
            Z2 = clf.predict_proba(np.c_[xx.ravel(), yy])

            for i in Z2:
                print(i)

            proba_df = pd.DataFrame(Z2)
            proba_df.columns = clf.classes_


            proba_list = []
            for i in range(len(proba_df)):
                proba_list.append(round(max(proba_df.iloc[i])+ 0.001, 2))
            predict_result = pd.concat(
                [self.data_to_test['Label'], pd.DataFrame({'SVM Classification': Z}), pd.DataFrame({'Confidence probability': proba_list})],
                axis=1).set_index('Label')
            print(predict_result)


            self.predictpop = TableViewer(df=predict_result, title='SVM Predict Result')
            self.predictpop.show()

            '''
            DataFileOutput, ok2 = QFileDialog.getSaveFileName(self, '文件保存', 'C:/',  'Excel Files (*.xlsx);;CSV Files (*.csv)')  # 数据文件保存输出
            if (DataFileOutput != ''):
                if ('csv' in DataFileOutput):
                    # DataFileOutput = DataFileOutput[0:-4]
                    predict_result.to_csv(DataFileOutput, sep=',', encoding='utf-8')
                    # self.result.to_csv(DataFileOutput + '.csv', sep=',', encoding='utf-8')
                elif ('xlsx' in DataFileOutput):
                    # DataFileOutput = DataFileOutput[0:-5]
                    predict_result.to_excel(DataFileOutput, encoding='utf-8')
                    # self.result.to_excel(DataFileOutput + '.xlsx', encoding='utf-8')
            '''

        except Exception as e:
            msg = 'You need to load another data to run SVM.\n '
            self.ErrorEvent(text= msg +repr(e) )

    def Explain(self):
        # self.OutPutData = self.OutPutData.set_index('Label')
        self.tablepop = TableViewer(df=self.OutPutData, title='Key_Func Result')
        self.tablepop.show()




from geopytool.ImportDependence import *
from geopytool.CustomClass import *
#from geopytool.TableViewer import TableViewer


class FluidInclusion(AppForm):
    _df = pd.DataFrame()
    _changed = False

    xlabel = r'Homogenization Temperature'
    ylabel = r'Frequency'

    reference = 'Wang Zhenyi Research on Fluid Inclusions of the Gejiu Tin-Copper Polymetallic Deposit 王振义. 个旧锡铜多金属矿床流体包裹体特征研究[D].'


    whole_labels = []
    all_labels = []
    all_colors = []
    all_markers = []
    all_alpha = []
    all_data_list = []


    def __init__(self, parent=None, df=pd.DataFrame(),filename= '/'):
        QWidget.__init__(self, parent)

        self.setWindowTitle('TAS (total alkali–silica) diagram Volcanic/Intrusive (Wilson et al. 1989)')
        self.FileName_Hint = ''
        self._df = df
        self.filename= filename

        if (len(df) > 0):
            self._changed = True
            # print('DataFrame recieved to AppForm')

        self.create_main_frame()
        self.create_status_bar()



    def create_main_frame(self):
        self.resize(1000, 800)
        self.main_frame = QWidget()
        self.dpi = 128
        self.fig = Figure((18.0, 12.0), dpi=self.dpi)
        self.fig.subplots_adjust(hspace=0.5, wspace=0.5, left=0.1, bottom=0.2, right=0.7, top=0.9)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)
        self.axes = self.fig.add_subplot(111)
        self.axes.axis('off')
        # Create the navigation toolbar, tied to the canvas
        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)

        # Other GUI controls


        self.save_button = QPushButton('&Save Img')
        self.save_button.clicked.connect(self.saveImgFile)

        self.stat_button = QPushButton('&Calculation')
        self.stat_button.clicked.connect(self.Explain)

        self.legend_cb = QCheckBox('&Legend')
        self.legend_cb.setChecked(True)
        self.legend_cb.stateChanged.connect(self.FluidInclusion)  # int


        self.density_cb = QCheckBox('&Density')
        self.density_cb.setChecked(True)
        self.density_cb.stateChanged.connect(self.FluidInclusion)  # int

        self.stack_cb = QCheckBox('&Stack')
        self.stack_cb.setChecked(True)
        self.stack_cb.stateChanged.connect(self.FluidInclusion)  # int


        self.combine_cb = QCheckBox('&Combine')
        self.combine_cb.setChecked(False)
        self.combine_cb.stateChanged.connect(self.FluidInclusion)  # int


        self.overlap_cb = QCheckBox('&Overlap')
        self.overlap_cb.setChecked(False)
        self.overlap_cb.stateChanged.connect(self.FluidInclusion)  # int


        self.hbox = QHBoxLayout()

        for w in [self.save_button,self.stat_button, self.legend_cb,self.density_cb,self.stack_cb,self.overlap_cb,self.combine_cb]:
            self.hbox.addWidget(w)
            self.hbox.setAlignment(w, Qt.AlignVCenter)

        self.vbox = QVBoxLayout()
        self.vbox.addWidget(self.mpl_toolbar)
        self.vbox.addWidget(self.canvas)
        self.vbox.addLayout(self.hbox)
        self.textbox = GrowingTextEdit(self)
        self.textbox.setText(self.reference)

        self.vbox.addWidget(self.textbox)

        self.main_frame.setLayout(self.vbox)
        self.setCentralWidget(self.main_frame)


    def FluidInclusion(self):
        self.setWindowTitle('Fluid Inclusion ')
        self.axes.clear()
        #self.axes.axis('off')
        self.axes.set_xlabel(self.xlabel)
        self.axes.set_ylabel(self.ylabel)
        ##self.axes.spines['top'].set_color('none')

        title = 'Fluid Inclusion '
        self.setWindowTitle(title)
        self.textbox.setText(self.reference)


        self.all_labels=[]
        self.all_colors=[]
        self.all_markers=[]
        self.all_alpha=[]
        self.all_data_list=[]


        if (self.combine_cb.isChecked()):

            self.all_data_list = self._df.Th


            if (self.density_cb.isChecked()):
                self.axes.hist(self.all_data_list,density=True, facecolor= 'grey', alpha= 0.6,
                           label=self.getFileName([self.filename]), edgecolor='k')
            else:
                self.axes.hist(self.all_data_list, density=False, facecolor= 'grey', alpha= 0.6,
                           label=self.getFileName([self.filename]), edgecolor='k')


        else:

            for i in range(len(self._df)):
                target = self._df.at[i, 'Label']
                color = self._df.at[i, 'Color']
                marker = self._df.at[i, 'Marker']
                alpha = self._df.at[i, 'Alpha']

                if target not in self.all_labels:
                    self.all_labels.append(target)
                    self.all_colors.append(color)
                    self.all_markers.append(marker)
                    self.all_alpha.append(alpha)

            self.whole_labels = self.all_labels

            for j in self.all_labels:
                tmp_data_list = []
                for i in range(len(self._df)):
                    target = self._df.at[i, 'Label']
                    if target == j:
                        tmp_data_list.append(self._df.at[i, 'Th'])

                self.all_data_list.append(tmp_data_list)

            if (self.stack_cb.isChecked()):
                pass

                if (self.density_cb.isChecked()):

                    self.axes.set_ylabel(self.ylabel+' Density')

                    N, bins, patches = self.axes.hist(self.all_data_list, density=True, stacked=True,edgecolor='k',alpha= 0.6)
                else:
                    N, bins, patches = self.axes.hist(self.all_data_list, density=False, stacked=True,edgecolor='k',alpha= 0.6)


                    #patches[i].set_facecolor('red')
                width = (bins[1] - bins[0]) * 0.4

                tmp_label_check=[]
                for k in range(len(patches)):
                    for p in patches[k]:
                        p.set_facecolor(self.all_colors[k])
                        p.set_alpha(self.all_alpha[k])

                        if self.all_labels[k] not in tmp_label_check:
                            tmp_label_check.append(self.all_labels[k])
                            p.set_label(self.all_labels[k])


            else:

                if (self.overlap_cb.isChecked()):
                    for k in range(len(self.all_labels)):

                        if (self.density_cb.isChecked()):

                            self.axes.set_ylabel(self.ylabel+' Density')

                            self.axes.hist(self.all_data_list[k], density=True, facecolor= self.all_colors[k], alpha= self.all_alpha[k],label=self.all_labels[k],edgecolor ='k')
                        else:
                            self.axes.hist(self.all_data_list[k], density=False, facecolor= self.all_colors[k], alpha= self.all_alpha[k],label=self.all_labels[k],edgecolor ='k')

                else:
                    if (self.density_cb.isChecked()):

                        self.axes.set_ylabel(self.ylabel+' Density')

                        N, bins, patches = self.axes.hist(self.all_data_list, density=True, stacked=False,edgecolor='k',alpha= 0.6)
                    else:
                        N, bins, patches = self.axes.hist(self.all_data_list, density=False, stacked=False,edgecolor='k',alpha= 0.6)


                        #patches[i].set_facecolor('red')

                    tmp_label_check=[]
                    for k in range(len(patches)):
                        for p in patches[k]:
                            p.set_facecolor(self.all_colors[k])
                            p.set_alpha(self.all_alpha[k])

                            if self.all_labels[k] not in tmp_label_check:
                                tmp_label_check.append(self.all_labels[k])
                                p.set_label(self.all_labels[k])



        if (self.legend_cb.isChecked()):
            self.axes.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0, prop=fontprop)

        self.canvas.draw()


    def Explain(self):

        #self.OutPutData = self.OutPutData.set_index('Label')

        self.OutPutData = pd.DataFrame()

        low_salt_list=[]
        high_salt_list=[]
        salt_list = []
        Th_list=[]
        Tm_list=[]



        salt_used_in_calc = []

        if 'Salinity(Wt%)'in self._df.columns.values.tolist():
            salt_used_in_calc= self._df['Salinity(Wt%)']
            self.OutPutData=self._df

            for i in range(len(self._df)):
                tmp = self._df.at[i, 'Tm']
                if abs(tmp) > 0:
                    Tm_list.append(tmp)
                    print(tmp)
                else:
                    Tm_list.append(np.NaN)
                tmp_th = self._df.at[i, 'Th']
                if abs(tmp_th) > 0:
                    Th_list.append(tmp_th)
                else:
                    Th_list.append(np.NaN)


        elif 'Salinity'in self._df.columns.values.tolist():
            salt_used_in_calc= self._df['Salinity']
            self.OutPutData=self._df
            for i in range(len(self._df)):
                tmp = self._df.at[i, 'Tm']
                if abs(tmp) > 0:
                    Tm_list.append(tmp)
                    print(tmp)
                else:
                    Tm_list.append(np.NaN)
                tmp_th = self._df.at[i, 'Th']
                if abs(tmp_th) > 0:
                    Th_list.append(tmp_th)
                else:
                    Th_list.append(np.NaN)
        else:
            for i in range(len(self._df)):
                tmp = self._df.at[i, 'Tm']
                if abs(tmp) > 0:
                    Tm_list.append(tmp)
                    print(tmp)
                    low_s = self.LowSalt(tmp)
                    high_s = self.HighSalt(tmp)
                    s = 0
                    if low_s <= 26.3:
                        s = low_s
                    else:
                        s = high_s

                    low_salt_list.append(round(low_s + 0.001, 2))
                    high_salt_list.append(round(high_s + 0.001, 2))
                    salt_list.append(round(s + 0.001, 2))
                else:
                    Tm_list.append(np.NaN)
                    low_salt_list.append(np.NaN)
                    high_salt_list.append(np.NaN)
                    salt_list.append(np.NaN)
                tmp_th = self._df.at[i, 'Th']
                if abs(tmp_th) > 0:
                    Th_list.append(tmp_th)
                else:
                    Th_list.append(np.NaN)

            SaltDict = {'Salinity(Wt%)': salt_list, 'LowSalinity(Wt%)': low_salt_list,
                        'HighSalinity(Wt%)': high_salt_list}
            SaltData = pd.DataFrame(SaltDict)

            self.OutPutData = pd.concat([self._df, SaltData], axis=1)
            salt_used_in_calc = salt_list

        density_list=[]
        p_ore_list=[]
        p_salt_list=[]
        depth_fluid_list=[]
        depth_host_list=[]
        for i in range(min(len(salt_used_in_calc),len(Th_list),len(Tm_list))):
            d_tmp=np.NaN
            po_tmp=np.NaN
            ps_tmp=np.NaN
            depth_fluid_tmp =np.NaN
            depth_host_rmp =np.NaN
            if abs(salt_used_in_calc[i]) >= 0 and abs(Th_list[i]) >= 0 and abs(Tm_list[i]) >= 0:
                d_tmp = self.DensitySalt(salt_used_in_calc[i],Th_list[i])
                po_tmp= self.PressureOre(salt_used_in_calc[i],Th_list[i])
                ps_tmp= self.PressureSalt(salt_used_in_calc[i],Th_list[i])

                depth_fluid_tmp =self.DepthFluid(po_tmp/10.0)
                depth_host_rmp =self.DepthHostRock(po_tmp/10.0)

            density_list.append(d_tmp)
            p_ore_list.append(po_tmp/10.0)
            p_salt_list.append(ps_tmp*np.power(10,2))

            depth_fluid_list.append(depth_fluid_tmp)
            depth_host_list.append(depth_host_rmp)


        CalcDict={'Liquid Density g/cm^3)':density_list,'Ore-forming Pressure(M Pa)':p_ore_list, 'Liquid Pressure(K Pa)':p_salt_list,'Depth by Fluid Pressure(km)':depth_fluid_list,'Depth of Host(km)':depth_host_list }
        CalcData = pd.DataFrame(CalcDict)

        self.OutPutData = pd.concat([self.OutPutData,CalcData], axis=1).set_index('Label')
        self.tablepop = TableViewer(df=self.OutPutData,title='Fluid Inclusion Calculation Result')
        self.tablepop.show()


    def LowSalt(self,Tm=0):
        # 低盐度流体(W(NaCl)< 23.3%,未见子矿物)的盐度计算公式, (Hall,1988)
        result= 0.00+1.78*abs(Tm)-0.0442*abs(Tm)*abs(Tm)+ 0.000557*abs(Tm)*abs(Tm)*abs(Tm)
        return(result)

    def HighSalt(self,Tm=0):
        # 高盐度流体(W(NaCl)> 26.3%,含有子矿物)的盐度计算公式, (Hall,1988;Sterner,1988)
        t = abs(Tm/1000.0)
        result = 26.242+ 0.4928*t+1.42*t*t-0.223*t*t*t +0.4129*t*t*t*t+0.006395*t*t*t*t*t-0.001967*t*t*t*t*t*t +0.0001112*t*t*t*t*t*t*t
        return (result)

    def CO2TripleSalt(self,Tmcla=0):
        # CO2三相流体包裹体的盐度计算公式 (Roedder,1984)
        result=0
        if -9/6<= Tmcla <=10.0:
            result = 15.52022-1.02342*Tmcla -0.05286*Tmcla*Tmcla
        return (result)

    def DensitySalt(self,W_NaCl,Th):
        # 不同浓度NaCl-H2O溶液密度公式(刘斌,1999,2001)
        w=W_NaCl
        t=Th
        A0 = 0.993531
        A1 = 8.72147 * (10**(-3))
        A2 = -2.43975 * (10**( -5))

        B0 = 7.11652 * (10**( -5))
        B1 = -5.2208 * (10**( -5))
        B2 = 1.26656 * (10**( -6))

        C0 = -3.4997 * (10**( -6))
        C1 = 2.12124 * (10**( -7))
        C2 = -4.52318 * (10**( -9))

        if 1<=w<30:
            # 1%~<30%时
            A0 = 0.993531
            A1 = 8.72147 * (10**( -3))
            A2 = -2.43975 * (10**( -5))

            B0 = 7.11652 * (10**( -5))
            B1 = -5.2208 * (10**( -5))
            B2 = 1.26656 * (10**( -6))

            C0 = -3.4997 * (10**( -6))
            C1 = 2.12124 * (10**( -7))
            C2 = -4.52318 * (10**( -9))

        elif 30<=w<60:

            # 30%~<60%时
            A0 = 1.376294
            A1 = 0.0106328
            A2 = -2.449428 * (10**( -4))

            B0 = -2.752237 * (10**( -3))
            B1 = 1.324187 * (10**( -5))
            B2 = 6.503339 * (10**( -7))

            C0 = 1.703392 * (10**( -6))
            C1 = -1.49158 * (10**( -8))
            C2 = -4.020795 * (10**( -10))

        elif 60<= w:
            # >60%时
            A0 = 21.31876
            A1 = -0.5715091
            A2 = 4.146964 * (10**( -3))

            B0 = -4.971499 * (10**( -2))
            B1 = 1.395219 * (10**( -3))
            B2 = -9.919401 * (10**( -6))

            C0 = 2.926059 * (10**( -5))
            C1 = -8.316942 * (10**( -7))
            C2 = 5.922209 * (10**( -9))


        A = A0 + A1 * w + A2 * w * w
        B = B0 + B1 * w + B2 * w * w
        C = C0 + C1 * w + C2 * w * w

        result = A + B * t + C * t * t
        return (result)

    def PressureOre(self,W_NaCl,Th):
        # 成矿压力经验公式, 邵洁涟, 1986
        result = 0
        w=W_NaCl
        Th=Th
        result = (219 + 2620 * w) * Th / (374 + 920 * w)
        # result 即压力, 单位是10^5 Pa
        return (result)

    def PressureSalt(self,W_NaCl,Th):
        # NaCl-H2O 溶液包裹体均一压力公式(Bain, 1964; Haas, 1976) 单位是bar, 即 10^5 Pa
        result = 0
        w=W_NaCl
        T=Th

        # 这里要先将NaCl的质量分数转换成摩尔浓度
        m=100*w/[58.4428 * (100-w)]

        e0 = 12.50849
        e1 = -4616.913
        e2 = 3.193455 * (10**( -4))
        e3 = 1.1965 * (10**( -11))
        e4 = -1.013137 * (10**( -2))
        e5 = -5.7148 * (10**( -3))
        em = np.log(10)
        A = 1 + 5.93582 * (10**( -6)) * m - 5.19386 * (10**( -5) )* m * m + 1.23156 * (10**( -5)) * m * m * m

        B = m * (1.1542 * (10**( -6)) + 1.4125 * (10**( -7) )* m - 1.92476 * (10**(-8)) * m * m - 1.70717 * (
            10**(-9)) * m * m * m + 1.0539 * (10**( -10)) * m * m * m * m)


        TH2O= np.power(np.e,np.log(T/(A+B*T)))

        z=TH2O+0.01

        Y= np.power(647.27-TH2O,1.25)

        delta = z*z -2.937* (10**(5))

        LnPg= e0+e1/z +e2*delta* (np.power(np.e, e3*em*delta*delta)- 1)/z+e4* np.power(10, e5*Y)

        result= np.power(np.e,LnPg)

        return (result)

    def PressureHighSalt(self,T):
        # NaCl-H2O 溶液包裹体有子晶的压力公式(Bischoff,1991) 温度范围在100-801℃
        # T 是NaCl子晶体最后消失的温度, 返回的P是压力, 单位是10^5 Pa
        t=T
        result=0
        result = 41.749 - 1.2125 * t + 0.0136213 * t * t - 7.52333 * (10**( -5)) * t * t * t + 2.19664 * (10**( -7)) * t * t * t * t - 2.82583 *(10**(-10)) * t * t * t * t * t + 1.27231 * (10**( -13)) * t * t * t * t * t * t

        return(result)


    def DepthFluid(self,Pressure_Ore_Forming=20):

        #断裂带流体压力与成矿深度之间的关系式（孙丰月，2000）
        #H 代表成矿深度（km），P 代表成矿压力（MPa）即 10^6 Pa。

        p=Pressure_Ore_Forming
        H=0

        if p <40:
            H=p/10.0
        elif 40<=p<220:
            H=(0.0868/(1/p+0.00388) +2)
        elif 220<=p<=370:
            H= 11+ (np.e**(p-221.95) )/79.075
        else: #p>370Mpa
            H=0.0331385*p+4.19898

        result = H

        return (result)

    def DepthHostRock(self,Pressure_Ore_Forming=20,rough=2.7,g=9.8):

        #静岩压力与深度的关系式 （孙丰月，2000）
        #P 为成矿压力（Mpa）即 10^6 Pa，rough 𝜌为上覆岩石的密度，取 2.7g/cm3,g 为重力加速度，取 9.8m/s2
        result = 0
        p=Pressure_Ore_Forming
        rough=rough
        g=g
        H=p/(rough*g)
        result=H
        return (result)



from geopytool.ImportDependence import *
#from geopytool.QAPF import *

class PandasModel(QtCore.QAbstractTableModel):
    _df = pd.DataFrame()
    _changed = False

    def __init__(self, df=pd.DataFrame(), parent=None):
        QtCore.QAbstractTableModel.__init__(self, parent=parent)
        self._df = df
        self._changed = False


        self._filters = {}
        self._sortBy = []
        self._sortDirection = []


    def headerData(self, section, orientation, role=QtCore.Qt.DisplayRole):
        if role != QtCore.Qt.DisplayRole:
            return QtCore.QVariant()

        if orientation == QtCore.Qt.Horizontal:
            try:
                return self._df.columns.tolist()[section]
            except (IndexError,):
                return QtCore.QVariant()
        elif orientation == QtCore.Qt.Vertical:
            try:
                # return self.df.index.tolist()
                return self._df.index.tolist()[section]
            except (IndexError,):
                return QtCore.QVariant()

    def data(self, index, role):
        if role == QtCore.Qt.DisplayRole or role == QtCore.Qt.EditRole:
            try:
                row = index.row()
                col = index.column()
                name = self._struct[col]['name']
                return self._data[row][name]
            except:
                pass
        elif role == QtCore.Qt.CheckStateRole:
            return None

        return QtCore.QVariant(str(self._df.ix[index.row(), index.column()]))

    def flags(self, index):
        return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEditable

    '''
    def setData(self, index, value, role=QtCore.Qt.EditRole):
        row = index.row()
        col = index.column()
        name = self._struct[col]['name']
        self._data[row][name] = value
        self.emit(QtCore.SIGNAL('dataChanged()'))
        return True
    '''

    def setData(self, index, value, role=QtCore.Qt.EditRole):
        row = self._df.index[index.row()]
        col = self._df.columns[index.column()]
        if hasattr(value, 'toPyObject'):
            # PyQt4 gets a QVariant
            value = value.toPyObject()
        else:
            # PySide gets an unicode
            dtype = self._df[col].dtype
            if dtype != object:
                value = None if value == '' else dtype.type(value)
        self._df.set_value(row, col, value)
        self._changed = True
        # self.emit(QtCore.SIGNAL('dataChanged()'))
        return True

    def rowCount(self, parent=QtCore.QModelIndex()):
        return len(self._df.index)

    def columnCount(self, parent=QtCore.QModelIndex()):
        return len(self._df.columns)

    def sort(self, column, order):
        colname = self._df.columns.tolist()[column]

        index = self._df.index.tolist()
        self.layoutAboutToBeChanged.emit()

        #self._df.sort_values(colname, ascending=order == QtCore.Qt.AscendingOrder, inplace=True)
        #self._df.reset_index(inplace=True, drop=True)

        try:
            self._df.sort_values(colname, ascending=order == QtCore.Qt.AscendingOrder, inplace=True)
        except:
            pass
        try:
            self._df.reset_index(inplace=True, drop=True)
        except:
            pass

        self.layoutChanged.emit()

class CustomQTableView(QtWidgets.QTableView):

    df = pd.DataFrame()
    def __init__(self, *args):
        super().__init__(*args)

        self.resize(800, 600)
        self.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers |
                             QtWidgets.QAbstractItemView.DoubleClicked)


    def keyPressEvent(self, event):  # Reimplement the event here
        return

class NewCustomQTableView(QtWidgets.QTableView):

    path=''
    def __init__(self, *args):
        super().__init__(*args)
        self.setAcceptDrops(True)

        self.resize(800, 600)
        self.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers |
                             QtWidgets.QAbstractItemView.DoubleClicked)

    def keyPressEvent(self, event):  # Reimplement the event here
        return


    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            files = [(u.toLocalFile()) for u in event.mimeData().urls()]
            for f in files:
                if 'csv' in f or 'xls' in f:
                    print('Drag', f)
                    self.path=f

                    if ('csv' in f):
                        self.parent().raw = pd.read_csv(f)
                    elif ('xls' in f):
                        self.parent().raw = pd.read_excel(f)

                    # #print(self.raw)


            event.accept()
        else:
            event.ignore()

    def dropEvent(self, event):
        files = [(u.toLocalFile()) for u in event.mimeData().urls()]
        for f in files:
            print('Drop')


class TableViewer(QMainWindow):
    addon = 'Name Author DataType Label Marker Color Size Alpha Style Width TOTAL total LOI loi'

    Minerals = ['Quartz',
                'Zircon',
                'K2SiO3',
                'Anorthite',
                'Na2SiO3',
                'Acmite',
                'Diopside',
                'Sphene',
                'Hypersthene',
                'Albite',
                'Orthoclase',
                'Wollastonite',
                'Olivine',
                'Perovskite',
                'Nepheline',
                'Leucite',
                'Larnite',
                'Kalsilite',
                'Apatite',
                'Halite',
                'Fluorite',
                'Anhydrite',
                'Thenardite',
                'Pyrite',
                'Magnesiochromite',
                'Chromite',
                'Ilmenite',
                'Calcite',
                'Na2CO3',
                'Corundum',
                'Rutile',
                'Magnetite',
                'Hematite',
                'Q',
                'A',
                'P',
                'F', ]

    Calced = ['Fe3+/(Total Fe) in rock',
              'Mg/(Mg+Total Fe) in rock',
              'Mg/(Mg+Fe2+) in rock',
              'Mg/(Mg+Fe2+) in silicates',
              'Ca/(Ca+Na) in rock',
              'Plagioclase An content',
              'Differentiation Index']
    DataWeight = {}
    DataVolume = {}
    DataBase = {}
    DataCalced = {}
    raw = pd.DataFrame()

    result = pd.DataFrame()
    Para = pd.DataFrame()
    _df = pd.DataFrame()
    data_to_test = pd.DataFrame()
    begin_result = pd.DataFrame()
    load_result = pd.DataFrame()

    def __init__(self, parent=None, df=pd.DataFrame(), title='Statistical Result'):
        QMainWindow.__init__(self, parent)
        self.setAcceptDrops(True)
        self.setWindowTitle(title)
        self.df = df
        self.create_main_frame()
        self.create_status_bar()

    def Old__init__(self, parent=None, df=pd.DataFrame()):
        QWidget.__init__(self, parent)
        self.setWindowTitle('TAS (total alkali–silica) diagram Volcanic/Intrusive (Wilson et al. 1989)')

        self._df = df
        if (len(df) > 0):
            self._changed = True
            # print('DataFrame recieved to TabelView')

        self.AllLabel = []

        for i in range(len(self._df)):
            tmp_label = self._df.at[i, 'Label']
            if tmp_label not in self.AllLabel:
                self.AllLabel.append(tmp_label)

        for i in range(len(self.LocationAreas)):
            tmpi = self.LocationAreas[i] + [self.LocationAreas[i][0]]
            tmppath = path.Path(tmpi)
            self.AreasHeadClosed.append(tmpi)
            patch = patches.PathPatch(tmppath, facecolor='orange', lw=0.3, alpha=0.3)

            self.SelectDic[self.ItemNames[i]] = tmppath

        self.create_main_frame()
        self.create_status_bar()

    def create_main_frame(self):

        self.resize(800, 600)
        self.main_frame = QWidget()

        self.save_button = QPushButton('&Save Result')
        self.save_button.clicked.connect(self.saveResult)

        self.tableView = CustomQTableView(self.main_frame)
        self.tableView.setObjectName('tableView')
        self.tableView.setSortingEnabled(True)

        self.vbox = QVBoxLayout()

        self.vbox.addWidget(self.tableView)
        self.vbox.addWidget(self.save_button)

        self.main_frame.setLayout(self.vbox)
        self.setCentralWidget(self.main_frame)

        self.model = PandasModel(self.df)
        self.tableView.setModel(self.model)

    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.accept()
        else:
            event.ignore()

    def dropEvent(self, event):
        files = [(u.toLocalFile()) for u in event.mimeData().urls()]
        for f in files:
            print(f)

    def clearLayout(self, layout):
        if layout is not None:
            while layout.count():
                item = layout.takeAt(0)
                widget = item.widget()
                if widget is not None:
                    widget.deleteLater()
                else:
                    self.clearLayout(item.layout())

    def resizeEvent(self, evt=None):

        w = self.width()
        h = self.height()
        '''
        if h<=360:
            h=360
            self.resize(w,h)
        if w<=640:
            w = 640
            self.resize(w, h)
        '''

        step = (w * 94 / 100) / 5
        foot = h * 3 / 48

    def ErrorEvent(self, text=''):

        _translate = QtCore.QCoreApplication.translate

        if (text == ''):
            reply = QMessageBox.information(self, _translate('MainWindow', 'Warning'), _translate('MainWindow',
                                                                                                  'Your Data mismatch this Function.\n Some Items missing?\n Or maybe there are blanks in items names?\n Or there are nonnumerical value？'))
        else:
            reply = QMessageBox.information(self, _translate('MainWindow', 'Warning'), _translate('MainWindow',
                                                                                                  'Your Data mismatch this Function.\n Error infor is:') + text)

    def create_main_frame(self):

        self.resize(800, 600)
        self.main_frame = QWidget()

        self.save_button = QPushButton('&Save Result')
        self.save_button.clicked.connect(self.saveResult)

        self.tableView = CustomQTableView(self.main_frame)
        self.tableView.setObjectName('tableView')
        self.tableView.setSortingEnabled(True)


        self.vbox = QVBoxLayout()

        self.vbox.addWidget(self.tableView)
        self.vbox.addWidget(self.save_button)

        self.main_frame.setLayout(self.vbox)
        self.setCentralWidget(self.main_frame)

        self.model = PandasModel(self.df)
        self.tableView.setModel(self.model)


    def create_status_bar(self):
        self.status_text = QLabel("Click Save button to save your figure.")
        self.statusBar().addWidget(self.status_text, 1)

    def add_actions(self, target, actions):
        for action in actions:
            if action is None:
                target.addSeparator()
            else:
                target.addAction(action)

    def saveDataFile(self):

        # if self.model._changed == True:
        # print('changed')
        # print(self.model._df)

        DataFileOutput, ok2 = QFileDialog.getSaveFileName(self,
                                                          '文件保存',
                                                          'C:/',
                                                          'Excel Files (*.xlsx);;CSV Files (*.csv)')  # 数据文件保存输出

        if "Label" in self.model._df.columns.values.tolist():
            self.model._df = self.model._df.set_index('Label')

        if (DataFileOutput != ''):

            if ('csv' in DataFileOutput):
                self.model._df.to_csv(DataFileOutput, sep=',', encoding='utf-8')

            elif ('xls' in DataFileOutput):
                self.model._df.to_excel(DataFileOutput, encoding='utf-8')

    def saveResult(self):

        self.result.reset_index
        DataFileOutput, ok2 = QFileDialog.getSaveFileName(self,
                                                          '文件保存',
                                                          'C:/',
                                                          'Excel Files (*.xlsx);;CSV Files (*.csv)')  # 数据文件保存输出

        if (DataFileOutput != ''):

            if ('csv' in DataFileOutput):

                # DataFileOutput = DataFileOutput[0:-4]

                self.result.to_csv(DataFileOutput, sep=',', encoding='utf-8')
                # self.result.to_csv(DataFileOutput + '.csv', sep=',', encoding='utf-8')

            elif ('xlsx' in DataFileOutput):

                # DataFileOutput = DataFileOutput[0:-5]

                self.result.to_excel(DataFileOutput, encoding='utf-8')

                # self.result.to_excel(DataFileOutput + '.xlsx', encoding='utf-8')

    def savePara(self):

        self.Para.reset_index
        DataFileOutput, ok2 = QFileDialog.getSaveFileName(self,
                                                          '文件保存',
                                                          'C:/',
                                                          'Excel Files (*.xlsx);;CSV Files (*.csv)')  # 数据文件保存输出

        if (DataFileOutput != ''):

            if ('csv' in DataFileOutput):

                # DataFileOutput = DataFileOutput[0:-4]

                self.Para.to_csv(DataFileOutput, sep=',', encoding='utf-8')
                # self.Para.to_csv(DataFileOutput + '.csv', sep=',', encoding='utf-8')

            elif ('xlsx' in DataFileOutput):

                # DataFileOutput = DataFileOutput[0:-5]

                self.Para.to_excel(DataFileOutput, encoding='utf-8')

                # self.Para.to_excel(DataFileOutput + '.xlsx', encoding='utf-8')

    def create_action(self, text, slot=None, shortcut=None,
                      icon=None, tip=None, checkable=False,
                      signal='triggered()'):
        action = QAction(text, self)
        if icon is not None:
            action.setIcon(QIcon(':/%s.png' % icon))
        if shortcut is not None:
            action.setShortcut(shortcut)
        if tip is not None:
            action.setToolTip(tip)
            action.setStatusTip(tip)
        if slot is not None:
            action.triggered.connect(slot)
        if checkable:
            action.setCheckable(True)
        return action


class OldTableViewer(QMainWindow):
    addon = 'Name Author DataType Label Marker Color Size Alpha Style Width TOTAL total LOI loi'

    Minerals = ['Quartz',
                'Zircon',
                'K2SiO3',
                'Anorthite',
                'Na2SiO3',
                'Acmite',
                'Diopside',
                'Sphene',
                'Hypersthene',
                'Albite',
                'Orthoclase',
                'Wollastonite',
                'Olivine',
                'Perovskite',
                'Nepheline',
                'Leucite',
                'Larnite',
                'Kalsilite',
                'Apatite',
                'Halite',
                'Fluorite',
                'Anhydrite',
                'Thenardite',
                'Pyrite',
                'Magnesiochromite',
                'Chromite',
                'Ilmenite',
                'Calcite',
                'Na2CO3',
                'Corundum',
                'Rutile',
                'Magnetite',
                'Hematite',
                'Q',
                'A',
                'P',
                'F', ]

    Calced = ['Fe3+/(Total Fe) in rock',
              'Mg/(Mg+Total Fe) in rock',
              'Mg/(Mg+Fe2+) in rock',
              'Mg/(Mg+Fe2+) in silicates',
              'Ca/(Ca+Na) in rock',
              'Plagioclase An content',
              'Differentiation Index']
    DataWeight = {}
    DataVolume = {}
    DataBase = {}
    DataCalced = {}
    raw = pd.DataFrame()

    def __init__(self, parent=None, df=pd.DataFrame(),title='Statistical Result'):
        QMainWindow.__init__(self, parent)
        self.setAcceptDrops(True)
        self.setWindowTitle(title)
        self.df = df
        self.create_main_frame()
        self.create_status_bar()

    def create_main_frame(self):

        self.resize(800, 600)
        self.main_frame = QWidget()

        self.save_button = QPushButton('&Save Result')
        self.save_button.clicked.connect(self.saveResult)

        self.tableView = CustomQTableView(self.main_frame)
        self.tableView.setObjectName('tableView')
        self.tableView.setSortingEnabled(True)


        self.vbox = QVBoxLayout()

        self.vbox.addWidget(self.tableView)
        self.vbox.addWidget(self.save_button)

        self.main_frame.setLayout(self.vbox)
        self.setCentralWidget(self.main_frame)

        self.model = PandasModel(self.df)
        self.tableView.setModel(self.model)




    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.accept()
        else:
            event.ignore()

    def dropEvent(self, event):
        files = [(u.toLocalFile()) for u in event.mimeData().urls()]
        for f in files:
            print(f)

    def saveResult(self):
        DataFileOutput, ok2 = QFileDialog.getSaveFileName(self,
                                                          '文件保存',
                                                          'C:/',
                                                          'Excel Files (*.xlsx);;CSV Files (*.csv)')  # 数据文件保存输出


        df = self.df
        if 'Label' in df.columns.values:
            df.set_index('Label', inplace=True)

        #self.model._df.reset_index(drop=True)


        if (DataFileOutput != ''):

            if ('csv' in DataFileOutput):
                df.to_csv(DataFileOutput, sep=',', encoding='utf-8')

            elif ('xls' in DataFileOutput):
                df.to_excel(DataFileOutput, encoding='utf-8')



from geopytool.ImportDependence import *
from geopytool.CustomClass import *


class MyDT(AppForm):
    Lines = []
    Tags = []
    WholeData = []
    settings_backup=pd.DataFrame()
    description = 'DecisionTree'
    unuseful = ['Name',
                'Mineral',
                'Author',
                'DataType',
                'Label',
                'Marker',
                'Color',
                'Size',
                'Alpha',
                'Style',
                'Width',
                'Tag']
    data_to_test =pd.DataFrame()
    switched = True
    text_result = ''
    whole_labels=[]
    clf = tree.DecisionTreeClassifier()
    #pca = DecisionTree(n_components=3)
    n=6

    def __init__(self, parent=None, df=pd.DataFrame()):
        QMainWindow.__init__(self, parent)
        self._df = df
        self._df_back = df

        self.FileName_Hint='Decision Tree'

        if (len(df) > 0):
            self._changed = True
            # print('DataFrame recieved to DecisionTree')



        self.settings_backup = self._df
        ItemsToTest = ['Label','Number', 'Tag', 'Name', 'Author', 'DataType', 'Marker', 'Color', 'Size', 'Alpha',
                       'Style', 'Width']
        for i in self._df.columns.values.tolist():
            if i not in ItemsToTest:
                self.settings_backup = self.settings_backup.drop(i, 1)
        #print(self.settings_backup)


        self.result_to_fit= self.Slim(self._df)


        #print(self.result_to_fit)
        #print(self.result_to_fit.shape)

        #self.pca=DecisionTree(n_components='mle')

        try:
            self.clf.fit(self.result_to_fit.values,self._df.Label)
        except Exception as e:
            self.ErrorEvent(text=repr(e))

        self.create_main_frame()

    def create_main_frame(self):
        self.resize(1200,800)
        self.main_frame = QWidget()
        self.dpi = 960
        self.fig = Figure((8.0, 6.0), dpi=self.dpi)
        self.setWindowTitle('Decision Tree Classifier')
        self.fig = plt.figure(figsize=(12, 6))
        #self.fig.subplots_adjust(hspace=0.5, wspace=0.5, left=0.1, bottom=0.1, right=0.9, top=0.9)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)


        self.legend_cb = QCheckBox('&Legend')
        self.legend_cb.setChecked(True)
        self.legend_cb.stateChanged.connect(self.Key_Func)  # int

        self.show_load_data_cb = QCheckBox('&Show Loaded Data')
        self.show_load_data_cb.setChecked(True)
        self.show_load_data_cb.stateChanged.connect(self.Key_Func)  # int

        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)

        # Other GUI controls
        self.save_picture_button = QPushButton('&Save Picture')
        self.save_picture_button.clicked.connect(self.saveImgFile)

        self.save_result_button = QPushButton('&Show DecisionTree Result')
        self.save_result_button.clicked.connect(self.showResult)

        self.save_Para_button = QPushButton('&Show DecisionTree Para')
        self.save_Para_button.clicked.connect(self.showPara)

        self.save_predict_button = QPushButton('&Show Predict Result')
        self.save_predict_button.clicked.connect(self.showPredictResult)

        self.load_data_button = QPushButton('&Load Data')
        self.load_data_button.clicked.connect(self.loadDataToTest)



        self.vbox = QVBoxLayout()
        self.hbox = QHBoxLayout()
        self.hbox0 = QHBoxLayout()
        self.hbox1 = QHBoxLayout()
        self.hbox2 = QHBoxLayout()
        self.hbox3 = QHBoxLayout()
        self.hbox4 = QHBoxLayout()


        self.vbox.addWidget(self.mpl_toolbar)
        self.vbox.addWidget(self.canvas)

        for w in [self.show_load_data_cb, ]:
            self.hbox0.addWidget(w)
            self.hbox0.setAlignment(w, Qt.AlignVCenter)

        for w in [self.load_data_button, self.save_picture_button, self.save_result_button, self.save_Para_button, self.save_predict_button]:
            self.hbox1.addWidget(w)
            self.hbox1.setAlignment(w, Qt.AlignVCenter)


        self.vbox.addLayout(self.hbox0)
        self.vbox.addLayout(self.hbox1)
        self.vbox.addLayout(self.hbox2)
        self.vbox.addLayout(self.hbox3)



        self.fig.subplots_adjust(hspace=0.5, wspace=0.5, left=0.1, bottom=0.2, right=0.7, top=0.9)
        self.axes = self.fig.add_subplot(111)


        self.main_frame.setLayout(self.vbox)
        self.setCentralWidget(self.main_frame)
        #self.show()

    def switch(self):
        self.switched = not(self.switched)
        self.create_main_frame()
        self.Key_Func()

    def Key_Func(self):

        self.axes.clear()
        self.clf.fit(self.result_to_fit.values,self._df.Label)


        self.Para=pd.DataFrame()

        self.data_to_test_to_fit=pd.DataFrame()


        if(len(self.data_to_test)>0):

            self.data_to_test_to_fit = self.Slim(self.data_to_test)

            contained = True
            missing = 'Miss setting infor:'

            for i in ['Label', 'Color', 'Marker', 'Alpha']:
                if i not in self.data_to_test.columns.values.tolist():
                    contained = False
                    missing = missing +'\n' + i

            if contained == True:
                for i in self.data_to_test.columns.values.tolist():
                    if i not in self._df.columns.values.tolist():
                        self.data_to_test=self.data_to_test.drop(columns=i)



                self.clf.predict(self.data_to_test_to_fit)
                self.clf.predict_proba(self.data_to_test_to_fit)



        if (self.legend_cb.isChecked()):

            if (self.switched == False):
                    self.axes.legend(loc=2, prop=fontprop)
            else:
                self.axes.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0, prop=fontprop)


        #self.result = pd.concat([self.begin_result , self.load_result], axis=0,sort=False).set_index('Label')

        tree.plot_tree(self.clf, max_depth=6, filled = True)
        self.canvas.draw()


    def showPredictResult(self):
        if(len(self.data_to_test)>0):
            Z = self.clf.predict(np.c_[ self.data_to_test_to_fit])
            Z2 = self.clf.predict_proba(np.c_[ self.data_to_test_to_fit])
            proba_df=pd.DataFrame(Z2)
            proba_df.columns = self.clf.classes_

            proba_list = []
            for i in range(len(proba_df)):
                proba_list.append(round(max(proba_df.iloc[i]) + 0.001, 2))
            predict_result = pd.concat(
                [self.data_to_test['Label'], pd.DataFrame({'Decision Tree Classification': Z}),
                 pd.DataFrame({'Confidence probability': proba_list}),proba_df],
                axis=1).set_index('Label')
            print(predict_result)

            self.predictpop = TableViewer(df=predict_result, title=self.description +' Decision Tree Predict Result with All Items')
            self.predictpop.show()




from geopytool.ImportDependence import *
from geopytool.CustomClass import *
from PyQt5 import QtGui  # (the example applies equally well to PySide)
import pyqtgraph as pg
import pandas as pd
import pyqtgraph.opengl as gl
import numpy as np


class GLMultiDimension(AppForm):
    Element = [u'Cs', u'Tl', u'Rb', u'Ba', u'W', u'Th', u'U', u'Nb', u'Ta', u'K', u'La', u'Ce', u'Pb', u'Pr', u'Mo',
               u'Sr', u'P', u'Nd', u'F', u'Sm', u'Zr', u'Hf', u'Eu', u'Sn', u'Sb', u'Ti', u'Gd', u'Tb', u'Dy',
               u'Li',
               u'Y', u'Ho', u'Er', u'Tm', u'Yb', u'Lu']

    StandardsName = ['OIB', 'EMORB', 'C1', 'PM', 'NMORB']

    NameChosen = 'OIB'
    Standards = {
        'OIB': {'Cs': 0.387, 'Tl': 0.077, 'Rb': 31, 'Ba': 350, 'W': 0.56, 'Th': 4, 'U': 1.02, 'Nb': 48, 'Ta': 2.7,
                'K': 12000, 'La': 37, 'Ce': 80, 'Pb': 3.2, 'Pr': 9.7, 'Mo': 2.4, 'Sr': 660, 'P': 2700, 'Nd': 38.5,
                'F': 1150, 'Sm': 10, 'Zr': 280, 'Hf': 7.8, 'Eu': 3, 'Sn': 2.7, 'Sb': 0.03, 'Ti': 17200, 'Gd': 7.62,
                'Tb': 1.05, 'Dy': 5.6, 'Li': 5.6, 'Y': 29, 'Ho': 1.06, 'Er': 2.62, 'Tm': 0.35, 'Yb': 2.16, 'Lu': 0.3},
        'EMORB': {'Cs': 0.063, 'Tl': 0.013, 'Rb': 5.04, 'Ba': 57, 'W': 0.092, 'Th': 0.6, 'U': 0.18, 'Nb': 8.3,
                  'Ta': 0.47, 'K': 2100, 'La': 6.3, 'Ce': 15, 'Pb': 0.6, 'Pr': 2.05, 'Mo': 0.47, 'Sr': 155, 'P': 620,
                  'Nd': 9, 'F': 250, 'Sm': 2.6, 'Zr': 73, 'Hf': 2.03, 'Eu': 0.91, 'Sn': 0.8, 'Sb': 0.01, 'Ti': 6000,
                  'Gd': 2.97, 'Tb': 0.53, 'Dy': 3.55, 'Li': 3.5, 'Y': 22, 'Ho': 0.79, 'Er': 2.31, 'Tm': 0.356,
                  'Yb': 2.37, 'Lu': 0.354},
        'C1': {'Cs': 0.188, 'Tl': 0.14, 'Rb': 2.32, 'Ba': 2.41, 'W': 0.095, 'Th': 0.029, 'U': 0.008, 'Nb': 0.246,
               'Ta': 0.014, 'K': 545, 'La': 0.237, 'Ce': 0.612, 'Pb': 2.47, 'Pr': 0.095, 'Mo': 0.92, 'Sr': 7.26,
               'P': 1220, 'Nd': 0.467, 'F': 60.7, 'Sm': 0.153, 'Zr': 3.87, 'Hf': 0.1066, 'Eu': 0.058, 'Sn': 1.72,
               'Sb': 0.16, 'Ti': 445, 'Gd': 0.2055, 'Tb': 0.0374, 'Dy': 0.254, 'Li': 1.57, 'Y': 1.57, 'Ho': 0.0566,
               'Er': 0.1655, 'Tm': 0.0255, 'Yb': 0.17, 'Lu': 0.0254},
        'PM': {'Cs': 0.032, 'Tl': 0.005, 'Rb': 0.635, 'Ba': 6.989, 'W': 0.02, 'Th': 0.085, 'U': 0.021, 'Nb': 0.713,
               'Ta': 0.041, 'K': 250, 'La': 0.687, 'Ce': 1.775, 'Pb': 0.185, 'Pr': 0.276, 'Mo': 0.063, 'Sr': 21.1,
               'P': 95, 'Nd': 1.354, 'F': 26, 'Sm': 0.444, 'Zr': 11.2, 'Hf': 0.309, 'Eu': 0.168, 'Sn': 0.17,
               'Sb': 0.005, 'Ti': 1300, 'Gd': 0.596, 'Tb': 0.108, 'Dy': 0.737, 'Li': 1.6, 'Y': 4.55, 'Ho': 0.164,
               'Er': 0.48, 'Tm': 0.074, 'Yb': 0.493, 'Lu': 0.074},
        'NMORB': {'Cs': 0.007, 'Tl': 0.0014, 'Rb': 0.56, 'Ba': 6.3, 'W': 0.01, 'Th': 0.12, 'U': 0.047, 'Nb': 2.33,
                  'Ta': 0.132, 'K': 600, 'La': 2.5, 'Ce': 7.5, 'Pb': 0.3, 'Pr': 1.32, 'Mo': 0.31, 'Sr': 90, 'P': 510,
                  'Nd': 7.3, 'F': 210, 'Sm': 2.63, 'Zr': 74, 'Hf': 2.05, 'Eu': 1.02, 'Sn': 1.1, 'Sb': 0.01, 'Ti': 7600,
                  'Gd': 3.68, 'Tb': 0.67, 'Dy': 4.55, 'Li': 4.3, 'Y': 28, 'Ho': 1.01, 'Er': 2.97, 'Tm': 0.456,
                  'Yb': 3.05, 'Lu': 0.455}, }

    Lines = []
    Tags = []

    xlabel = 'x'
    ylabel = 'y'
    zlabel = 'z'

    description = 'X-Y- diagram'
    unuseful = ['Name',
                'Author',
                'DataType',
                'Label',
                'Marker',
                'Color',
                'Size',
                'Alpha',
                'Style',
                'Width',
                'Type',
                'Tag']

    width_plot = 100.0
    height_plot = 100.0
    depth_plot= 100.0

    width_load = width_plot
    height_load = height_plot
    depth_load = depth_plot

    polygon = []
    polyline = []
    line = []

    strgons = []
    strlines = []
    strpolylines = []

    extent = 0

    Left = 0
    Right = 0
    Up = 0
    Down = 0

    FitLevel=3
    FadeGroups=100
    ShapeGroups=200

    Xleft,Xright,Ydown,Yup,Ztail,Zhead=0,0,0,0,0,0

    LimSet= False

    def __init__(self, parent=None, df=pd.DataFrame()):
        QMainWindow.__init__(self, parent)
        self.setWindowTitle(self.description)

        self.items = []

        self._df = df
        if (len(df) > 0):
            self._changed = True
            # print('DataFrame recieved to GLMultiDimension')

        self.raw = df
        self.rawitems = self.raw.columns.values.tolist()

        for i in self.rawitems:
            if i not in self.unuseful:
                self.items.append(i)
            else:
                pass

        self.create_main_frame()
        self.create_status_bar()

        self.polygon = 0
        self.polyline = 0

        self.flag = 0

    def create_main_frame(self):
        self.main_frame = QWidget()

        #self.main_frame.setFixedSize(self.width(), self.width())

        self.dpi = 128
        self.ShapeGroups =200
        self.view = gl.GLViewWidget()

        #self.view = pg.PlotWidget()

        #self.view.setFixedSize(self.width(),self.height())

        self.view.setFixedSize(self.width(), self.width())

        self.view.setParent(self.main_frame)


        # Other GUI controls
        self.save_button = QPushButton('&Save')
        self.save_button.clicked.connect(self.saveImgFile)

        self.draw_button = QPushButton('&Reset')
        self.draw_button.clicked.connect(self.Reset)

        self.load_button = QPushButton('&Load')
        #self.load_button.clicked.connect(self.Load)


        self.fit_cb= QCheckBox('&PolyFit')
        self.fit_cb.setChecked(False)
        self.fit_cb.stateChanged.connect(self.Magic)  # int

        self.fit_label = QLabel('Exp')
        self.fit_seter = QLineEdit(self)
        self.fit_seter.textChanged[str].connect(self.FitChanged)


        self.shape_cb= QCheckBox('&Shape')
        self.shape_cb.setChecked(False)
        self.shape_cb.stateChanged.connect(self.Magic)  # int


        self.Normalize_cb = QCheckBox('&Normalize')
        self.Normalize_cb.setChecked(False)
        self.Normalize_cb.stateChanged.connect(self.Magic)  # int

        self.norm_slider_label = QLabel('Standard:' + self.NameChosen)
        self.norm_slider = QSlider(Qt.Horizontal)
        self.norm_slider.setRange(0, 4)
        self.norm_slider.setValue(0)
        self.norm_slider.setTracking(True)
        self.norm_slider.setTickPosition(QSlider.TicksBothSides)
        self.norm_slider.valueChanged.connect(self.Magic)  # int

        self.x_element = QSlider(Qt.Horizontal)
        self.x_element.setRange(0, len(self.items) - 1)
        self.x_element.setValue(0)
        self.x_element.setTracking(True)
        self.x_element.setTickPosition(QSlider.TicksBothSides)
        self.x_element.valueChanged.connect(self.Magic)  # int

        self.x_element_label = QLabel('X')

        self.logx_cb = QCheckBox('&Log')
        self.logx_cb.setChecked(False)
        self.logx_cb.stateChanged.connect(self.Magic)  # int

        self.y_element = QSlider(Qt.Horizontal)
        self.y_element.setRange(0, len(self.items) - 1)
        self.y_element.setValue(1)
        self.y_element.setTracking(True)
        self.y_element.setTickPosition(QSlider.TicksBothSides)
        self.y_element.valueChanged.connect(self.Magic)  # int

        self.y_element_label = QLabel('Y')

        self.logy_cb = QCheckBox('&Log')
        self.logy_cb.setChecked(False)
        self.logy_cb.stateChanged.connect(self.Magic)  # int

        self.z_element = QSlider(Qt.Horizontal)
        self.z_element.setRange(0, len(self.items) - 1)
        self.z_element.setValue(2)
        self.z_element.setTracking(True)
        self.z_element.setTickPosition(QSlider.TicksBothSides)
        self.z_element.valueChanged.connect(self.Magic)  # int

        self.z_element_label = QLabel('Z')

        self.logz_cb = QCheckBox('&Log')
        self.logz_cb.setChecked(False)
        self.logz_cb.stateChanged.connect(self.Magic)  # int




        self.xlim_seter_left_label = QLabel('Xleft')
        self.xlim_seter_left = QLineEdit(self)
        self.xlim_seter_left.textChanged[str].connect(self.XleftChanged)

        self.xlim_seter_right_label = QLabel('Xright')
        self.xlim_seter_right = QLineEdit(self)
        self.xlim_seter_right.textChanged[str].connect(self.XrightChanged)


        self.ylim_seter_down_label = QLabel('Ydown')
        self.ylim_seter_down = QLineEdit(self)
        self.ylim_seter_down.textChanged[str].connect(self.YdownChanged)


        self.ylim_seter_up_label = QLabel('Yup')
        self.ylim_seter_up = QLineEdit(self)
        self.ylim_seter_up.textChanged[str].connect(self.YupChanged)



        self.hbox0 = QHBoxLayout()
        self.hbox1 = QHBoxLayout()
        self.hbox2 = QHBoxLayout()
        self.hbox3 = QHBoxLayout()
        self.hbox4 = QHBoxLayout()
        self.hbox5 = QHBoxLayout()
        self.hbox6 = QHBoxLayout()
        self.hbox7 = QHBoxLayout()



        '''
        for w in [self.fit_cb,self.fit_label, self.fit_seter,self.xlim_seter_left_label,self.xlim_seter_left,self.xlim_seter_right_label,self.xlim_seter_right,self.ylim_seter_down_label,self.ylim_seter_down,self.ylim_seter_up_label,self.ylim_seter_up,self.shape_cb]:
            self.hbox0.addWidget(w)
            self.hbox0.setAlignment(w, Qt.AlignVCenter)
        '''

        for w in [self.view]:
            self.hbox0.addWidget(w)
            self.hbox0.setAlignment(w, Qt.AlignVCenter)


        for w in [self.Normalize_cb, self.norm_slider_label, self.norm_slider]:
            self.hbox1.addWidget(w)
            self.hbox1.setAlignment(w, Qt.AlignVCenter)

        for w in [self.logx_cb, self.x_element_label, self.x_element]:
            self.hbox2.addWidget(w)
            self.hbox2.setAlignment(w, Qt.AlignVCenter)

        for w in [self.logy_cb, self.y_element_label, self.y_element]:
            self.hbox3.addWidget(w)
            self.hbox3.setAlignment(w, Qt.AlignVCenter)

        for w in [self.logz_cb, self.z_element_label, self.z_element]:
            self.hbox4.addWidget(w)
            self.hbox4.setAlignment(w, Qt.AlignVCenter)



        self.vbox = QVBoxLayout()
        #self.vbox.addWidget(self.view)
        self.vbox.addLayout(self.hbox0)
        self.vbox.addLayout(self.hbox1)
        self.vbox.addLayout(self.hbox2)
        self.vbox.addLayout(self.hbox3)
        self.vbox.addLayout(self.hbox4)

        self.main_frame.setLayout(self.vbox)
        self.setCentralWidget(self.main_frame)

    def Read(self, inpoints):
        points = []
        for i in inpoints:
            points.append(i.split())

        result = []
        for i in points:
            for l in range(len(i)):
                a = float((i[l].split(','))[0])
                a = a * self.x_scale

                b = float((i[l].split(','))[1])
                b = (self.height_load - b) * self.y_scale

                result.append((a, b))
        return (result)

    def Load(self):
        fileName, filetype = QFileDialog.getOpenFileName(self,
                                                         '选取文件',
                                                         '~/',
                                                         'PNG Files (*.png);;JPG Files (*.jpg);;SVG Files (*.svg)')  # 设置文件扩展名过滤,注意用双分号间隔

        print(fileName, '\t', filetype)

        if ('svg' in fileName):
            doc = minidom.parse(fileName)  # parseString also exists
            polygon_points = [path.getAttribute('points') for path in doc.getElementsByTagName('polygon')]
            polyline_points = [path.getAttribute('points') for path in doc.getElementsByTagName('polyline')]

            svg_width = [path.getAttribute('width') for path in doc.getElementsByTagName('svg')]
            svg_height = [path.getAttribute('height') for path in doc.getElementsByTagName('svg')]

            # print(svg_width)
            # print(svg_height)

            digit = '01234567890.-'
            width = svg_width[0].replace('px', '').replace('pt', '')
            height = svg_height[0].replace('px', '').replace('pt', '')

            self.width_load = float(width)
            self.height_load = float(height)

            soup = BeautifulSoup(open(fileName), 'lxml')

            tmpgon = soup.find_all('polygon')
            tmppolyline = soup.find_all('polyline')
            tmptext = soup.find_all('text')
            tmpline = soup.find_all('line')

            tmppath = soup.find_all('path')

            self.strgons = []
            for i in tmpgon:
                a = (str(i)).replace('\n', '').replace('\t', '')
                m = BeautifulSoup(a, 'lxml')
                k = m.polygon.attrs
                self.strgons.append(k['points'].split())

            self.strpolylines = []
            for i in tmppolyline:
                a = (str(i)).replace('\n', '').replace('\t', '')
                m = BeautifulSoup(a, 'lxml')
                k = m.polyline.attrs
                self.strpolylines.append(k['points'].split())

            self.strlines = []
            for i in tmpline:
                a = (str(i)).replace('\n', '').replace('\t', '')
                m = BeautifulSoup(a, 'lxml')
                k = m.line.attrs
                a = str(k['x1']) + ',' + str(k['y1']) + ' ' + str(k['x2']) + ',' + str(k['y2'])
                self.strlines.append(a.split())

            self.strpath = []
            for i in tmppath:
                a = (str(i)).replace('\n', '').replace('\t', '')
                m = BeautifulSoup(a, 'lxml')
                k = m.path.attrs
                self.strpath.append(k['d'].split())

            # print(self.strpath)



            self.polygon = []
            for i in self.strgons:
                m = self.Read(i)
                m.append(m[0])
                self.polygon.append(m)

            self.polyline = []
            for i in self.strpolylines:
                m = self.Read(i)
                # print('i: ',i,'\n m:',m)
                self.polyline.append(m)

            self.line = []
            for i in self.strlines:
                m = self.Read(i)
                # print('i: ',i,'\n m:',m)
                self.line.append(m)


        elif ('png' in fileName or 'jpg' in fileName):

            self.img = mpimg.imread(fileName)
            self.flag = 1

        self.Magic()

    def Reset(self):
        self.flag = 0
        self.Magic()

    def FitChanged(self, text):
        w = 'Fit' + text
        self.fit_label.setText(w)
        self.fit_label.adjustSize()

        try:
            self.FitLevel = float(text)
        except:
            pass

        self.Magic()


    def XleftChanged(self,text):
        if len(text)<1:
            self.LimSet = False
        else:
            self.LimSet = True
            w = 'Left ' + text
            self.xlim_seter_left_label.setText(w)
            self.xlim_seter_left_label.adjustSize()

            try:
                self.Xleft = float(text)
            except:
                pass

            self.Magic()

    def XrightChanged(self,text):
        if len(text)<1:
            self.LimSet = False
        else:
            self.LimSet = True
            w = 'Right ' + text
            self.xlim_seter_right_label.setText(w)
            self.xlim_seter_right_label.adjustSize()

            try:
                self.Xright = float(text)
            except:
                pass

            self.Magic()

    def YdownChanged(self,text):
        if len(text)<1:
            self.LimSet = False
        else:
            self.LimSet = True
            w = 'Down ' + text
            self.ylim_seter_down_label.setText(w)
            self.ylim_seter_down_label.adjustSize()

            try:
                self.Ydown = float(text)
            except:
                pass

            self.Magic()

    def YupChanged(self,text):
        if len(text)<1:
            self.LimSet = False
        else:
            self.LimSet =True
            w = 'Up ' + text
            self.ylim_seter_up_label.setText(w)
            self.ylim_seter_up_label.adjustSize()

            try:
                self.Yup = float(text)
            except:
                pass

            self.Magic()


    def ShapeChanged(self, text):
        w = 'Shape' + text
        #self.shape_label.setText(w)
        #self.shape_label.adjustSize()

        try:
            self.ShapeGroups = int(text)
        except:
            pass

        self.Magic()

    def GetASequence(self, head=0, tail= 200, count=10):

        if count > 0:
            result = np.arange(head, tail, (tail - head) / count)
        else:
            result = np.arange(head, tail, (tail - head) / 10)

        return (result)

    def Magic(self):

        #self.view.setFixedSize(self.width(), self.width())

        self.WholeData = []

        self.x_scale = self.width_plot / self.width_load

        self.y_scale = self.height_plot / self.height_load

        self.z_scale = self.depth_plot / self.depth_load


        # print(self.x_scale,' and ',self.x_scale)

        raw = self._df

        a = int(self.x_element.value())

        b = int(self.y_element.value())

        c = int(self.z_element.value())

        self.x_element_label.setText(self.items[a])
        self.y_element_label.setText(self.items[b])
        self.z_element_label.setText(self.items[c])


        if (self.Left != self.Right) and (self.Down != self.Up) and abs(self.Left) + abs(self.Right) + abs(
                self.Down) + abs(self.Up) != 0:
            self.extent = [self.Left, self.Right, self.Down, self.Up]
        elif (self.Left == self.Right and abs(self.Left) + abs(self.Right) != 0):
            reply = QMessageBox.warning(self, 'Warning', 'You set same value to Left and Right limits.')
            self.extent = 0

        elif (self.Down == self.Up and abs(self.Down) + abs(self.Up) != 0):
            reply = QMessageBox.warning(self, 'Warning', 'You set same value to Up and Down limits.')
            self.extent = 0
        else:
            self.extent = 0

        standardnamechosen = self.StandardsName[int(self.norm_slider.value())]
        standardchosen = self.Standards[standardnamechosen]

        self.norm_slider_label.setText(standardnamechosen)


        PointLabels = []
        XtoDraw = []
        YtoDraw = []
        ZtoDraw = []
        Colors=[]
        Alphas=[]
        Markers=[]
        Names=[]

        for i in range(len(raw)):
            # raw.at[i, 'DataType'] == 'User' or raw.at[i, 'DataType'] == 'user' or raw.at[i, 'DataType'] == 'USER'

            TmpLabel = ''

            #   self.WholeData.append(math.log(tmp, 10))

            if (raw.at[i, 'Label'] in PointLabels or raw.at[i, 'Label'] == ''):
                TmpLabel = ''
            else:
                PointLabels.append(raw.at[i, 'Label'])
                TmpLabel = raw.at[i, 'Label']

            x, y ,z = 0, 0, 0
            xuse, yuse,zuse = 0, 0, 0

            x, y,z = raw.at[i, self.items[a]], raw.at[i, self.items[b]],raw.at[i, self.items[c]]


            try:
                xuse = x
                yuse = y
                zuse = z

                self.xlabel = self.items[a]
                self.ylabel = self.items[b]
                self.zlabel = self.items[c]

                if (self.Normalize_cb.isChecked()):

                    self.xlabel = self.items[a] + ' Norm by ' + standardnamechosen
                    self.x_element_label.setText(self.xlabel)

                    self.ylabel = self.items[b] + ' Norm by ' + standardnamechosen
                    self.y_element_label.setText(self.ylabel)

                    self.zlabel = self.items[c] + ' Norm by ' + standardnamechosen
                    self.z_element_label.setText(self.zlabel)


                    if self.items[a] in self.Element:
                        xuse = xuse / standardchosen[self.items[a]]

                    if self.items[b] in self.Element:
                        yuse = yuse / standardchosen[self.items[b]]

                    if self.items[c] in self.Element:
                        zuse = zuse / standardchosen[self.items[c]]

                if (self.logx_cb.isChecked()):
                    xuse = math.log(x, 10)
                    self.xlabel = '$log10$ ' + self.xlabel


                if (self.logy_cb.isChecked()):
                    yuse = math.log(y, 10)

                    self.ylabel = '$log10$ ' + self.ylabel

                if (self.logz_cb.isChecked()):
                    zuse = math.log(z, 10)

                    self.zlabel = '$log10$ ' + self.zlabel

                XtoDraw.append(xuse)
                YtoDraw.append(yuse)
                ZtoDraw.append(zuse)
                Colors.append(raw.at[i, 'Color'])
                Alphas.append(raw.at[i, 'Alpha'])
                Names.append(raw.at[i, 'Label'])
                Markers.append(raw.at[i, 'Marker'])


            except(ValueError):
                pass


        if self.LimSet==False:
            self.Xleft, self.Xright, self.Ydown, self.Yup, self.Tail, self.Head = min(XtoDraw), max(XtoDraw), min(YtoDraw), max(YtoDraw), min(ZtoDraw), max(ZtoDraw)





        xmin, xmax = min(XtoDraw), max(XtoDraw)
        ymin, ymax = min(YtoDraw), max(YtoDraw)
        zmin, zmax = min(ZtoDraw), max(ZtoDraw)

        xmean = np.mean(XtoDraw)
        ymean = np.mean(YtoDraw)
        zmean = np.mean(ZtoDraw)

        Xoriginal = np.arange(xmin, xmax, (xmax - xmin) / 10)
        Yoriginal = np.arange(ymin, ymax, (ymax - ymin) / 10)
        Zoriginal = np.arange(zmin, zmax, (zmax - zmin) / 10)

        XonPlot = self.GetASequence(tail=self.ShapeGroups)
        YonPlot = self.GetASequence(tail=self.ShapeGroups)
        ZonPlot = self.GetASequence(tail=self.ShapeGroups)

        XonStick = []
        YonStick = []
        ZonStick = []

        for i in range(len(XonPlot)):
            XonStick.append([XonPlot[i], Xoriginal[i]])
            YonStick.append([YonPlot[i], Yoriginal[i]])
            ZonStick.append([ZonPlot[i], Zoriginal[i]])
            pass




        #print(XtoDraw,'\n', YtoDraw,'\n', ZtoDraw)

        toDf = {self.xlabel:XtoDraw,
                self.ylabel:YtoDraw,
                self.zlabel:ZtoDraw}




        newdf = pd.DataFrame(toDf)

        pos = newdf.as_matrix()

        print(pos)

        ThreeDimView = gl.GLScatterPlotItem(pos=pos, color=(100, 255, 255, 88), size=0.1, pxMode=False)


        print(xmean,'\n', ymean,'\n', zmean,'\n')

        self.view.pan(xmean, ymean, zmean)


        xgrid = gl.GLGridItem(size=QtGui.QVector3D(10, 10, 1), color=1)
        ygrid = gl.GLGridItem(size=QtGui.QVector3D(20, 20, 2), color=2)
        zgrid = gl.GLGridItem(size=QtGui.QVector3D(30, 30, 3), color=3)

        ## rotate x and y grids to face the correct direction
        xgrid.rotate(90, 0, 1, 0)
        ygrid.rotate(90, 1, 0, 0)



        xgrid.translate(xmean, ymean, zmean)
        ygrid.translate(xmean, ymean, zmean)
        zgrid.translate(xmean, ymean, zmean)


        ## scale each grid differently

        '''
        xgrid.scale(12.8, 12.8, 12.8)
        ygrid.scale(12.8, 12.8, 12.8)
        zgrid.scale(12.8, 12.8, 12.8)
        '''

        # xgrid.setTransform(xmean,ymean,zmean)

        self.view.addItem(xgrid)
        self.view.addItem(ygrid)
        self.view.addItem(zgrid)
        self.view.addItem(ThreeDimView)
